# Stubs for docutils.utils.math.math2html (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils.utils.math import math2html
from gettext import NullTranslations
from typing import Any, Callable, ClassVar, Dict, Iterator, List, Mapping, MutableSequence, Optional, Sequence, TextIO, Tuple, Type, TypeVar, Union

class Trace:
    debugmode: bool = ...
    quietmode: bool = ...
    showlinesmode: bool = ...
    prefix: ClassVar[Optional[str]] = ...
    def debug(cls: Any, message: str) -> None: ...
    def message(cls: Any, message: str) -> None: ...
    def error(cls: Any, message: str) -> None: ...
    def fatal(cls: Any, message: str) -> None: ...
    def show(cls: Any, message: str, channel: TextIO) -> None: ...
    debug: classmethod = ...  # type: ignore  # NOQA: F811
    message: classmethod = ...  # type: ignore  # NOQA: F811
    error: classmethod = ...  # type: ignore  # NOQA: F811
    fatal: classmethod = ...  # type: ignore  # NOQA: F811
    show: classmethod = ...  # type: ignore  # NOQA: F811

class BibStylesConfig:
    abbrvnat: Dict[str, str] = ...
    alpha: Dict[str, str] = ...
    authordate2: Dict[str, str] = ...
    default: Dict[str, str] = ...
    defaulttags: Dict[str, str] = ...
    ieeetr: Dict[str, str] = ...
    plain: Dict[str, str] = ...
    vancouver: Dict[str, str] = ...

class BibTeXConfig:
    replaced: Dict[str, str] = ...

class ContainerConfig:
    endings: Dict[str, str] = ...
    extracttext: Dict[str, str] = ...
    startendings: Dict[str, str] = ...
    starts: Dict[str, str] = ...
    string: Dict[str, str] = ...
    table: Dict[str, List[str]] = ...

class EscapeConfig:
    chars: Dict[str, str] = ...
    commands: Dict[str, str] = ...
    entities: Dict[str, str] = ...
    html: Dict[str, str] = ...
    iso885915: Dict[str, str] = ...
    nonunicode: Dict[str, str] = ...

class FormulaConfig:
    alphacommands: Dict[str, str] = ...
    array: Dict[str, str] = ...
    bigbrackets: Dict[str, List[str]] = ...
    bigsymbols: Dict[str, List[str]] = ...
    bracketcommands: Dict[str, str] = ...
    combiningfunctions: Dict[str, str] = ...
    commands: Dict[str, str] = ...
    decoratedcommand: Dict[Any, Any] = ...
    decoratingfunctions: Dict[str, str] = ...
    endings: Dict[str, str] = ...
    environments: Dict[str, List[str]] = ...
    fontfunctions: Dict[str, str] = ...
    hybridfunctions: Dict[str, List[str]] = ...
    hybridsizes: Dict[str, str] = ...
    labelfunctions: Dict[str, str] = ...
    limitcommands: Dict[str, str] = ...
    misccommands: Dict[str, str] = ...
    modified: Dict[str, str] = ...
    onefunctions: Dict[str, str] = ...
    spacedcommands: Dict[str, str] = ...
    starts: Dict[str, str] = ...
    symbolfunctions: Dict[str, str] = ...
    textfunctions: Dict[str, str] = ...
    unmodified: Dict[str, List[str]] = ...
    urls: Dict[str, str] = ...

class GeneralConfig:
    version: Dict[str, str] = ...

class HeaderConfig:
    parameters: Dict[str, str] = ...
    styles: Dict[str, List[str]] = ...

class ImageConfig:
    converters: Dict[str, str] = ...
    cropboxformats: Dict[str, str] = ...
    formats: Dict[str, Union[str, List[str]]] = ...

class LayoutConfig:
    groupable: Dict[str, List[str]] = ...

class NewfangleConfig:
    constants: Dict[str, str] = ...

class NumberingConfig:
    layouts: Dict[str, List[str]] = ...
    sequence: Dict[str, List[str]] = ...

class StyleConfig:
    hspaces: Dict[str, str] = ...
    quotes: Dict[str, str] = ...
    referenceformats: Dict[str, str] = ...
    size: Dict[str, List[str]] = ...
    vspaces: Dict[str, str] = ...

class TOCConfig:
    extractplain: Dict[str, List[str]] = ...
    extracttitle: Dict[str, List[str]] = ...

class TagConfig:
    barred: Dict[str, str] = ...
    family: Dict[str, str] = ...
    flex: Dict[str, str] = ...
    group: Dict[str, List[str]] = ...
    layouts: Dict[str, str] = ...
    listitems: Dict[str, str] = ...
    notes: Dict[str, str] = ...
    script: Dict[str, str] = ...
    shaped: Dict[str, str] = ...

class TranslationConfig:
    constants: Dict[str, str] = ...
    languages: Dict[str, str] = ...

class CommandLineParser:
    options: Type[math2html.Options] = ...
    def __init__(self, options: Type[math2html.Options]) -> None: ...
    def parseoptions(self, args: Sequence[str]) -> Optional[str]: ...
    def readoption(self, args: Sequence[str]) -> Tuple[str, Union[str, bool, None]]: ...
    def readquoted(self, args: Sequence[str], initial: str) -> Optional[str]: ...
    def readequalskey(self, arg: str, args: MutableSequence[str]) -> str: ...

class Options:
    instance: Any = ...
    location: str = ...
    nocopy: bool = ...
    copyright: bool = ...
    debug: bool = ...
    quiet: bool = ...
    version: bool = ...
    hardversion: bool = ...
    versiondate: bool = ...
    html: bool = ...
    help: bool = ...
    showlines: bool = ...
    str: bool = ...
    iso885915: bool = ...
    css: List[Any] = ...
    favicon: __builtins__.str = ...
    title: Any = ...
    directory: Any = ...
    destdirectory: Any = ...
    toc: bool = ...
    toctarget: __builtins__.str = ...
    tocfor: Any = ...
    forceformat: Any = ...
    lyxformat: bool = ...
    target: Any = ...
    splitpart: Any = ...
    memory: bool = ...
    lowmem: bool = ...
    nobib: bool = ...
    converter: __builtins__.str = ...
    raw: bool = ...
    jsmath: Any = ...
    mathjax: Any = ...
    nofooter: bool = ...
    simplemath: bool = ...
    template: Any = ...
    noconvert: bool = ...
    notoclabels: bool = ...
    letterfoot: bool = ...
    numberfoot: bool = ...
    symbolfoot: bool = ...
    hoverfoot: bool = ...
    marginfoot: bool = ...
    endfoot: bool = ...
    supfoot: bool = ...
    alignfoot: bool = ...
    footnotes: Optional[__builtins__.str] = ...
    imageformat: Optional[__builtins__.str] = ...
    copyimages: bool = ...
    googlecharts: bool = ...
    embedcss: List[Any] = ...
    branches: Dict[__builtins__.str, math2html.BranchOptions] = ...
    def parseoptions(self, args: MutableSequence[__builtins__.str]) -> None: ...
    def processoptions(self) -> None: ...
    def usage(self) -> None: ...
    def parsefootnotes(self) -> None: ...
    def showoptions(self) -> None: ...
    def showversion(self) -> None: ...
    def showhardversion(self) -> None: ...
    def showversiondate(self) -> None: ...
    def showlyxformat(self) -> None: ...

class BranchOptions:
    name: str = ...
    options: Dict[str, str] = ...
    def __init__(self, name: str) -> None: ...
    def set(self, key: str, value: str) -> None: ...
    def isselected(self) -> bool: ...
    def __unicode__(self) -> str: ...

_Class = TypeVar('_Class')

class Cloner:
    def clone(cls: Any, original: _Class) -> _Class: ...
    def create(cls: Any, type: Type[_Class]) -> _Class: ...
    clone: classmethod = ...  # type: ignore  # NOQA: F811
    create: classmethod = ...  # type: ignore  # NOQA: F811

_Container = TypeVar('_Container', bound=math2html.Container)

class ContainerExtractor:
    allowed: List[str] = ...
    cloned: List[str] = ...
    extracted: List[str] = ...
    def __init__(self, config: Mapping[str, MutableSequence[str]]) -> None: ...
    def extract(self, container: math2html.Container) -> List[math2html.Container]: ...
    def process(self, container: math2html.Container, list: MutableSequence[math2html.Container]) -> None: ...
    def safeclone(self, container: _Container) -> _Container: ...

class Parser:
    begin: int = ...
    parameters: Dict[str, Union[Dict[str, str], bool, str]] = ...
    def __init__(self) -> None: ...
    def parseheader(self, reader: math2html.LineReader) -> List[str]: ...
    def parseparameter(self, reader: math2html.LineReader) -> None: ...
    def parsexml(self, reader: math2html.LineReader) -> Tuple[Optional[str], Optional[Dict[str, str]]]: ...
    def parseending(self, reader: math2html.LineReader, process: Callable[..., None]) -> None: ...
    def parsecontainer(self, reader: math2html.LineReader, contents: MutableSequence[math2html.Container]) -> None: ...
    def __unicode__(self) -> str: ...

class LoneCommand(Parser):
    def parse(self, reader: math2html.LineReader) -> List[Any]: ...

class TextParser(Parser):
    stack: List[str] = ...
    ending: str = ...
    endings: List[str] = ...
    def __init__(self, container: math2html.Container) -> None: ...
    def parse(self, reader: math2html.LineReader) -> List[math2html.Container]: ...
    def isending(self, reader: math2html.LineReader) -> bool: ...

class ExcludingParser(Parser):
    def parse(self, reader: math2html.LineReader) -> List[math2html.Container]: ...

class BoundedParser(ExcludingParser):
    def parse(self, reader: math2html.LineReader) -> List[math2html.Container]: ...

class BoundedDummy(Parser):
    def parse(self, reader: math2html.LineReader) -> List[math2html.Container]: ...

class StringParser(Parser):
    begin: int = ...
    def parseheader(self, reader: math2html.LineReader) -> List[Any]: ...
    def parse(self, reader: math2html.LineReader) -> List[math2html.Container]: ...

class InsetParser(BoundedParser):
    def parse(self, reader: math2html.LineReader) -> List[math2html.Container]: ...

class ContainerOutput:
    def gethtml(self, container: math2html.Container) -> Any: ...
    def isempty(self) -> bool: ...

class EmptyOutput(ContainerOutput):
    def gethtml(self, container: math2html.Container) -> List[Any]: ...
    def isempty(self) -> bool: ...

class FixedOutput(ContainerOutput):
    def gethtml(self, container: math2html.Container) -> List[str]: ...

class ContentsOutput(ContainerOutput):
    def gethtml(self, container: math2html.Container) -> List[str]: ...

class TaggedOutput(ContentsOutput):
    tag: Optional[str] = ...
    breaklines: bool = ...
    empty: bool = ...
    def settag(self, tag: str, breaklines: bool = ..., empty: bool = ...) -> math2html.TaggedOutput: ...
    def setbreaklines(self, breaklines: bool) -> math2html.TaggedOutput: ...
    def gethtml(self, container: math2html.Container) -> List[str]: ...
    def open(self, container: math2html.Container) -> str: ...
    def close(self, container: math2html.Container) -> str: ...
    def selfclosing(self, container: math2html.Container) -> str: ...
    def checktag(self) -> bool: ...

class FilteredOutput(ContentsOutput):
    filters: List[Tuple[str, str]] = ...
    def __init__(self) -> None: ...
    def addfilter(self, original: str, replacement: str) -> None: ...
    def gethtml(self, container: math2html.Container) -> List[str]: ...
    def filter(self, line: str) -> str: ...

class StringOutput(ContainerOutput):
    def gethtml(self, container: math2html.Container) -> List[str]: ...

class LineReader:
    file: TextIO = ...
    linenumber: int = ...
    lastline: Optional[int] = ...
    current: Optional[str] = ...
    mustread: bool = ...
    depleted: bool = ...
    def __init__(self, filename: Union[TextIO, str]) -> None: ...
    def setstart(self, firstline: int) -> None: ...
    def setend(self, lastline: Optional[int]) -> None: ...
    def currentline(self) -> Optional[str]: ...
    def nextline(self) -> None: ...
    def readline(self) -> None: ...
    def finished(self) -> bool: ...
    def close(self) -> None: ...

class LineWriter:
    file: bool = ...
    filename: Optional[str] = ...
    def __init__(self, filename: Union[TextIO, str]) -> None: ...
    def write(self, strings: Sequence[str]) -> None: ...
    def writestring(self, string: str) -> None: ...
    def writeline(self, line: str) -> None: ...
    def close(self) -> None: ...

class Globable:
    leavepending: bool = ...
    endinglist: math2html.EndingList = ...
    def __init__(self) -> None: ...
    def checkbytemark(self) -> None: ...
    def isout(self) -> bool: ...
    def current(self) -> str: ...
    def checkfor(self, string: str) -> bool: ...
    def finished(self) -> bool: ...
    def skipcurrent(self) -> str: ...
    def glob(self, currentcheck: Callable[[], bool]) -> str: ...
    def globalpha(self) -> str: ...
    def globnumber(self) -> str: ...
    def isidentifier(self) -> bool: ...
    def globidentifier(self) -> str: ...
    def isvalue(self) -> bool: ...
    def globvalue(self) -> str: ...
    def skipspace(self) -> str: ...
    def globincluding(self, magicchar: str) -> str: ...
    def globexcluding(self, excluded: str) -> str: ...
    def pushending(self, ending: str, optional: bool = ...) -> None: ...
    def popending(self, expected: Optional[str] = ...) -> Optional[str]: ...
    def nextending(self) -> Optional[str]: ...

class EndingList:
    endings: List[math2html.PositionEnding] = ...
    def __init__(self) -> None: ...
    def add(self, ending: str, optional: bool = ...) -> None: ...
    def pickpending(self, pos: math2html.Position) -> None: ...
    def checkin(self, pos: math2html.Position) -> bool: ...
    def pop(self, pos: math2html.Position) -> str: ...
    def findending(self, pos: math2html.Position) -> Optional[str]: ...
    def checkpending(self) -> None: ...
    def __unicode__(self) -> str: ...

class PositionEnding:
    ending: str = ...
    optional: bool = ...
    def __init__(self, ending: str, optional: bool) -> None: ...
    def checkin(self, pos: math2html.Position) -> bool: ...
    def __unicode__(self) -> str: ...

class Position(Globable):
    def __init__(self) -> None: ...
    def skip(self, string: str) -> None: ...
    def identifier(self) -> str: ...
    def extract(self, length: int) -> Optional[str]: ...
    def checkfor(self, string: str) -> bool: ...
    def checkforlower(self, string: str) -> bool: ...
    def skipcurrent(self) -> str: ...
    def __next__(self) -> str: ...
    def checkskip(self, string: str) -> bool: ...
    def error(self, message: str) -> None: ...

class TextPosition(Position):
    pos: int = ...
    text: str = ...
    def __init__(self, text: str) -> None: ...
    def skip(self, string: str) -> None: ...
    def identifier(self) -> str: ...
    def isout(self) -> bool: ...
    def current(self) -> str: ...
    def extract(self, length: int) -> Optional[str]: ...

class FilePosition(Position):
    reader: LineReader = ...
    pos: int = ...
    def __init__(self, filename: Union[TextIO, str]) -> None: ...
    def skip(self, string: str) -> None: ...
    def currentline(self) -> Optional[str]: ...
    def nextline(self) -> None: ...
    def linenumber(self) -> int: ...
    def identifier(self) -> str: ...
    def isout(self) -> bool: ...
    def current(self) -> str: ...
    def extract(self, length: int) -> str: ...

class Container:
    partkey: Any = ...
    parent: math2html.Container = ...
    begin: Any = ...
    contents: List[Any] = ...
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    def gethtml(self) -> List[str]: ...
    def escapeall(self, lines: Sequence[str]) -> List[str]: ...
    def escape(self, line: str, replacements: Dict[str, str] = ...) -> str: ...
    def escapeentities(self, line: str) -> str: ...
    def searchall(self, type: Type) -> List[math2html.Container]: ...
    def searchremove(self, type: Type) -> List[math2html.Container]: ...
    def searchprocess(self, type: Type, process: Callable[[math2html.Container], List[math2html.Container]]) -> None: ...
    def locateprocess(self, locate: Callable[[math2html.Container], bool], process: Callable[[math2html.Container], List[math2html.Container]]) -> None: ...
    def recursivesearch(self, locate: Callable[[math2html.Container], bool], recursive: Callable[[math2html.Container], bool], process: Callable[[math2html.Container], None]) -> None: ...
    def extracttext(self) -> str: ...
    def group(self, index: int, group, isingroup: Callable[[Any], bool]) -> None: ...
    def remove(self, index: int) -> None: ...
    def tree(self, level: int = ...) -> None: ...
    def getparameter(self, name: str) -> str: ...
    def getparameterlist(self, name: str) -> List[str]: ...
    def hasemptyoutput(self) -> bool: ...
    def __unicode__(self) -> str: ...

class BlackBox(Container):
    parser: LoneCommand = ...
    output: EmptyOutput = ...
    contents: List[Any] = ...
    def __init__(self) -> None: ...

class LyXFormat(BlackBox):
    def process(self) -> None: ...

class StringContainer(Container):
    parsed: Any = ...
    parser: StringParser = ...
    output: StringOutput = ...
    string: str = ...
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    def replacespecial(self, line: str) -> str: ...
    def changeline(self, line: str) -> str: ...
    def extracttext(self) -> str: ...
    def __unicode__(self) -> str: ...

class Constant(StringContainer):
    contents: List[Any] = ...
    string: str = ...
    output: StringOutput = ...
    def __init__(self, text: str) -> None: ...
    def __unicode__(self) -> str: ...

class TaggedText(Container):
    output: ContentsOutput = ...
    parser: TextParser = ...
    def __init__(self) -> None: ...
    contents: List[Constant] = ...
    def complete(self, contents: List[Constant], tag: str, breaklines: bool = ...): ...
    def constant(self, text: str, tag: str, breaklines: bool = ...): ...
    def __unicode__(self) -> str: ...

class DocumentParameters:
    pdftitle: Any = ...
    indentstandard: bool = ...
    tocdepth: int = ...
    startinglevel: int = ...
    maxdepth: int = ...
    language: str = ...
    bibliography: Any = ...
    outputchanges: bool = ...
    displaymode: bool = ...

class FormulaParser(Parser):
    begin: int = ...
    def parseheader(self, reader: LineReader): ...
    def parsetype(self, reader: LineReader) -> Optional[str]: ...
    def parse(self, reader: LineReader) -> str: ...
    def parseformula(self, reader: LineReader) -> str: ...
    def parsesingleliner(self, reader: LineReader, start: str, ending: str) -> str: ...
    def parsemultiliner(self, reader: LineReader, start: str, ending: str) -> str: ...

class MacroParser(FormulaParser):
    begin: int = ...
    def parseheader(self, reader: LineReader) -> List[str]: ...
    def parse(self, reader: LineReader) -> str: ...

class FormulaBit(Container):
    type: str = ...
    size: int = ...
    original: str = ...
    contents: List[Any] = ...
    output: ContentsOutput = ...
    def __init__(self) -> None: ...
    factory: math2html.FormulaFactory = ...
    def setfactory(self, factory: math2html.FormulaFactory) -> math2html.FormulaBit: ...
    def add(self, bit: math2html.FormulaBit) -> None: ...
    def skiporiginal(self, string: str, pos: Position) -> None: ...
    def computesize(self) -> int: ...
    def clone(self) -> math2html.FormulaBit: ...
    def __unicode__(self) -> str: ...

class TaggedBit(FormulaBit):
    output: TaggedOutput = ...
    def constant(self, constant: str, tag: str) -> math2html.TaggedBit: ...
    contents: List[FormulaBit] = ...
    def complete(self, contents: List[FormulaBit], tag: str, breaklines: bool = ...) -> math2html.TaggedBit: ...
    def selfcomplete(self, tag: str) -> math2html.TaggedBit: ...

class FormulaConstant(Constant):
    original: str = ...
    size: int = ...
    type: str = ...
    def __init__(self, string: str) -> None: ...
    def computesize(self) -> int: ...
    def clone(self) -> math2html.FormulaConstant: ...
    def __unicode__(self) -> str: ...

class RawText(FormulaBit):
    def detect(self, pos: Position) -> bool: ...
    type: str = ...
    def parsebit(self, pos: Position) -> None: ...

class FormulaSymbol(FormulaBit):
    modified: Dict[str, str] = ...
    unmodified: List[str] = ...
    def detect(self, pos: Position) -> bool: ...
    def parsebit(self, pos: Position) -> None: ...
    def addsymbol(self, symbol, pos: Position) -> None: ...

class FormulaNumber(FormulaBit):
    def detect(self, pos: Position) -> bool: ...
    type: str = ...
    def parsebit(self, pos: Position) -> None: ...

class Comment(FormulaBit):
    start: str = ...
    def detect(self, pos: Position) -> bool: ...
    def parsebit(self, pos: Position) -> None: ...

class WhiteSpace(FormulaBit):
    def detect(self, pos: Position) -> bool: ...
    def parsebit(self, pos: Position) -> None: ...
    def __unicode__(self) -> str: ...

class Bracket(FormulaBit):
    start: str = ...
    ending: str = ...
    inner: Any = ...
    def __init__(self) -> None: ...
    def detect(self, pos: Position) -> bool: ...
    def parsebit(self, pos: Position) -> math2html.Bracket: ...
    def parsetext(self, pos: Position) -> math2html.Bracket: ...
    def parseliteral(self, pos: Position) -> math2html.Bracket: ...
    def parsecomplete(self, pos: Position, innerparser: Callable[[Position], None]) -> None: ...
    def innerformula(self, pos: Position) -> None: ...
    def innertext(self, pos: Position) -> None: ...
    literal: str = ...
    def innerliteral(self, pos: Position) -> None: ...

class SquareBracket(Bracket):
    start: str = ...
    ending: str = ...
    def clone(self) -> math2html.SquareBracket: ...

class MathsProcessor:
    def process(self, contents, index) -> None: ...
    def __unicode__(self) -> str: ...

class FormulaProcessor:
    processors: List[MathsProcessor] = ...
    def process(self, bit: FormulaBit) -> None: ...
    def processcontents(self, bit: FormulaBit) -> None: ...
    def processinsides(self, bit: FormulaBit) -> None: ...
    def traversewhole(self, formula: FormulaBit) -> None: ...
    def traverse(self, bit: FormulaBit) -> Iterator[Tuple[FormulaBit, List[FormulaBit]]]: ...
    def italicize(self, bit: FormulaBit, contents) -> None: ...

class Formula(Container):
    parser: FormulaParser = ...
    output: TaggedOutput = ...
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    contents: List[Any] = ...
    def jsmath(self) -> None: ...
    def mathjax(self) -> None: ...
    def googlecharts(self) -> None: ...
    def classic(self) -> None: ...
    def parse(self, pos: Position) -> math2html.Formula: ...
    header: List[str] = ...
    def parsedollarinline(self, pos: Position) -> None: ...
    def parsedollarblock(self, pos: Position) -> None: ...
    parsed: str = ...
    def parsedollar(self, pos: Position) -> None: ...
    def parseinlineto(self, pos: Position, limit: str) -> None: ...
    def parseblockto(self, pos: Position, limit: str) -> None: ...
    def parseupto(self, pos: Position, limit: str) -> None: ...
    def __unicode__(self) -> str: ...

class WholeFormula(FormulaBit):
    def detect(self, pos: Position) -> bool: ...
    def parsebit(self, pos: Position) -> None: ...

_FB = TypeVar('_FB', bound=FormulaBit)

class FormulaFactory:
    types: List[Type[FormulaBit]] = ...
    skippedtypes: List[Type[FormulaBit]] = ...
    defining: bool = ...
    instances: Dict[Type[FormulaBit], FormulaBit] = ...
    def __init__(self) -> None: ...
    def detecttype(self, type: Type[_FB], pos: Position) -> _FB: ...
    def instance(self, type: Type[_FB]) -> _FB: ...
    def create(self, type: Type[_FB]) -> _FB: ...
    def clearskipped(self, pos: Position) -> None: ...
    def skipany(self, pos: Position) -> Optional[FormulaBit]: ...
    def parseany(self, pos: Position) -> FormulaConstant: ...
    def parsetype(self, type: Type[_FB], pos: Position) -> _FB: ...
    def parseformula(self, formula: str) -> WholeFormula: ...

class Translator:
    instance: math2html.Translator = ...
    def translate(cls: Any, key: str): ...
    translate: classmethod = ...  # type: ignore  # NOQA: F811
    translation: NullTranslations = ...
    first: bool = ...
    def __init__(self) -> None: ...
    langcodes: List[str] = ...
    def findtranslation(self) -> None: ...
    def getmessage(self, key: str) -> str: ...
    def getuntranslated(self, key: str) -> str: ...

class NumberCounter:
    name: str = ...
    value: int = ...
    mode: str = ...
    master: math2html.NumberCounter = ...
    letters: str = ...
    symbols: List[str] = ...
    romannumerals: List[Tuple[str, int]] = ...
    def __init__(self, name: str) -> None: ...
    def setmode(self, mode: str) -> math2html.NumberCounter: ...
    def init(self, value: int) -> None: ...
    def gettext(self) -> str: ...
    def getletter(self) -> str: ...
    def getsymbol(self) -> str: ...
    def getsequence(self, sequence: Union[str, MutableSequence[str]]) -> str: ...
    def getroman(self) -> str: ...
    def getvalue(self) -> str: ...
    def getnext(self) -> int: ...
    def reset(self) -> None: ...
    def __unicode__(self) -> str: ...

class DependentCounter(NumberCounter):
    master: NumberCounter = ...
    last: str = ...
    def setmaster(self, master: NumberCounter) -> math2html.DependentCounter: ...
    def getnext(self) -> int: ...
    def getvalue(self) -> str: ...

class NumberGenerator:
    chaptered: math2html.ChapteredGenerator = ...
    generator: math2html.NumberGenerator = ...
    romanlayouts: List[str] = ...
    orderedlayouts: List[str] = ...
    counters: Dict[str, NumberCounter] = ...
    appendix: Optional[bool] = ...
    def deasterisk(self, type: str) -> str: ...
    def isunique(self, type: str) -> bool: ...
    def isroman(self, type: str) -> bool: ...
    def isinordered(self, type: str) -> bool: ...
    def isnumbered(self, type: str) -> bool: ...
    def isunordered(self, type: str) -> bool: ...
    def getlevel(self, type: str) -> int: ...
    def getparttype(self, type: str) -> str: ...
    def generate(self, type: str) -> int: ...
    def getcounter(self, type: str) -> NumberCounter: ...
    def create(self, type: str) -> NumberCounter: ...
    def getdependentcounter(self, type: str, master: NumberCounter) -> NumberCounter: ...
    def createdependent(self, type: str, master: NumberCounter) -> DependentCounter: ...
    def startappendix(self) -> None: ...

class ChapteredGenerator(NumberGenerator):
    def generate(self, type: str) -> int: ...

class ContainerSize:
    width: Optional[str] = ...
    height: Optional[str] = ...
    maxwidth: Optional[str] = ...
    maxheight: Optional[str] = ...
    scale: Any = ...
    def set(self, width: Optional[str] = ..., height: Optional[str] = ...) -> math2html.ContainerSize: ...
    def setmax(self, maxwidth: Optional[str] = ..., maxheight: Optional[str] = ...) -> math2html.ContainerSize: ...
    def readparameters(self, container: Container) -> math2html.ContainerSize: ...
    def setparameter(self, container: Container, name: str) -> None: ...
    def setvalue(self, name: str, value: str) -> None: ...
    def checkvalidheight(self, container: Container) -> None: ...
    def processparameter(self, value: str) -> Optional[str]: ...
    def extractnumber(self, text: str) -> str: ...
    def checkimage(self, width: int, height: int) -> None: ...
    def scalevalue(self, value: int) -> str: ...
    def removepercentwidth(self) -> Optional[str]: ...
    def addstyle(self, container: Container) -> None: ...
    def styleparameter(self, name: str) -> str: ...

class QuoteContainer(Container):
    parser: BoundedParser = ...
    output: FixedOutput = ...
    def __init__(self) -> None: ...
    type: str = ...
    html: List[str] = ...
    def process(self) -> None: ...

class LyXLine(Container):
    parser: LoneCommand = ...
    output: FixedOutput = ...
    def __init__(self) -> None: ...
    html: List[str] = ...
    def process(self) -> None: ...

class EmphaticText(TaggedText):
    def process(self) -> None: ...

class ShapedText(TaggedText):
    type: str = ...
    def process(self) -> None: ...

class VersalitasText(TaggedText):
    def process(self) -> None: ...

class ColorText(TaggedText):
    color: str = ...
    def process(self) -> None: ...

class SizeText(TaggedText):
    size: str = ...
    def process(self) -> None: ...

class BoldText(TaggedText):
    def process(self) -> None: ...

class TextFamily(TaggedText):
    type: str = ...
    def process(self) -> None: ...

class Hfill(TaggedText):
    def process(self) -> None: ...

class BarredText(TaggedText):
    type: str = ...
    def process(self) -> None: ...

class LangLine(TaggedText):
    output: ContentsOutput = ...
    def process(self) -> None: ...

class InsetLength(BlackBox):
    length: str = ...
    def process(self) -> None: ...

class Space(Container):
    parser: InsetParser = ...
    output: FixedOutput = ...
    def __init__(self) -> None: ...
    type: str = ...
    html: List[str] = ...
    def process(self) -> None: ...
    def getlength(self) -> Optional[str]: ...

class VerticalSpace(Container):
    parser: InsetParser = ...
    output: FixedOutput = ...
    def __init__(self) -> None: ...
    type: str = ...
    html: List[str] = ...
    def process(self) -> None: ...

class Align(Container):
    parser: ExcludingParser = ...
    output: TaggedOutput = ...
    def __init__(self) -> None: ...
    def process(self) -> None: ...

class Newline(Container):
    parser: LoneCommand = ...
    output: FixedOutput = ...
    def __init__(self) -> None: ...
    html: List[str] = ...
    def process(self) -> None: ...

class NewPage(Newline):
    html: List[str] = ...
    def process(self) -> None: ...

class Separator(Container):
    output: FixedOutput = ...
    contents: List[Any] = ...
    html: List[str] = ...
    def __init__(self, constant: str) -> None: ...

class StrikeOut(TaggedText):
    def process(self) -> None: ...

class StartAppendix(BlackBox):
    def process(self) -> None: ...

class Link(Container):
    anchor: str = ...
    url: str = ...
    type: str = ...
    page: str = ...
    target: str = ...
    destination: math2html.Link = ...
    title: str = ...
    contents: List[Constant] = ...
    parser: InsetParser = ...
    output: math2html.LinkOutput = ...
    def __init__(self) -> None: ...
    def complete(self, text: str, anchor: Optional[str] = ..., url: Optional[str] = ..., type: Optional[str] = ..., title: Optional[str] = ...) -> math2html.Link: ...
    def computedestination(self) -> None: ...
    def setmutualdestination(self, destination: math2html.Link) -> None: ...
    def __unicode__(self) -> str: ...

class URL(Link):
    url: str = ...
    contents: List[Constant] = ...
    def process(self) -> None: ...

class FlexURL(URL):
    url: str = ...
    def process(self) -> None: ...

class LinkOutput(ContainerOutput):
    def gethtml(self, link: Link) -> List[str]: ...  # type: ignore

class Postprocessor:
    stages: math2html.StageDict = ...
    current: Container = ...
    last: Container = ...
    def __init__(self) -> None: ...
    def postprocess(self, next: Optional[List[Any]]) -> Any: ...
    def postrecursive(self, container: Container) -> None: ...
    def postcurrent(self, next: Optional[List[Any]]) -> Any: ...

class StageDict:
    stagedict: Dict[Any, Any] = ...
    def __init__(self, classes: Sequence[Type], postprocessor: Postprocessor) -> None: ...
    def instantiate(self, classes: Sequence[Type], postprocessor: Postprocessor) -> List[Any]: ...
    def getstage(self, element: Any) -> Optional[Any]: ...

class Label(Link):
    names: ClassVar[Dict[str, math2html.Label]] = ...
    lastlayout: Any = ...
    lastnumbered: Any = ...
    def __init__(self) -> None: ...
    def process(self) -> None: ...
    key: str = ...
    def create(self, text: str, key: str, type: str = ...) -> math2html.Label: ...
    def findpartkey(self) -> str: ...
    def numbered(self, container: Container) -> Optional[Container]: ...
    def __unicode__(self) -> str: ...

class Reference(Link):
    references: ClassVar[Dict[str, MutableSequence[math2html.Reference]]] = ...
    key: str = ...
    direction: str = ...
    destination: Label = ...
    def process(self) -> None: ...
    formatted: str = ...
    contents: List[Constant] = ...
    def formatcontents(self) -> None: ...
    def replace(self, key: str, value: str) -> None: ...
    def __unicode__(self) -> str: ...

_CB = TypeVar('_CB', bound=math2html.CommandBit)

class FormulaCommand(FormulaBit):
    types: List[Type] = ...
    start: str = ...
    commandmap: Optional[Mapping] = ...
    def detect(self, pos: Position) -> bool: ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> Optional[_CB]: ...
    def parsewithcommand(self, command: str, pos: Position) -> Optional[_CB]: ...
    def parsecommandtype(self, command: str, type: _CB, pos: Position) -> _CB: ...
    def extractcommand(self, pos: Position) -> Optional[str]: ...
    def emptycommand(self, pos: Position) -> str: ...
    def parseupgreek(self, command: str, pos: Position) -> Optional[_CB]: ...

class CommandBit(FormulaCommand):
    command: Any = ...
    translated: Any = ...
    def setcommand(self, command) -> None: ...
    def parseparameter(self, pos: Position) -> FormulaConstant: ...
    def parsesquare(self, pos: Position) -> Optional[_FB]: ...
    def parseliteral(self, pos: Position) -> str: ...
    def parsesquareliteral(self, pos: Position) -> str: ...
    def parsetext(self, pos: Position) -> Bracket: ...

class EmptyCommand(CommandBit):
    commandmap: Dict[str, str] = ...
    contents: List[FormulaConstant] = ...
    def parsebit(self, pos: Position) -> None: ...

class SpacedCommand(CommandBit):
    commandmap: Dict[str, str] = ...
    contents: List[FormulaConstant] = ...
    def parsebit(self, pos: Position) -> None: ...

class AlphaCommand(EmptyCommand):
    commandmap: Dict[str, str] = ...
    type: str = ...
    def parsebit(self, pos: Position) -> None: ...

class OneParamFunction(CommandBit):
    commandmap: Dict[str, str] = ...
    simplified: bool = ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...
    html: List[str] = ...
    def simplifyifpossible(self) -> None: ...

class SymbolFunction(CommandBit):
    commandmap: Dict[str, str] = ...
    def detect(self, pos: Position) -> bool: ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...

class TextFunction(CommandBit):
    commandmap: Dict[str, str] = ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...
    type: str = ...
    def process(self) -> None: ...

class LabelFunction(CommandBit):
    commandmap: Dict[str, str] = ...
    key: Any = ...
    def parsebit(self, pos: Position) -> None: ...
    type: str = ...
    label: Label = ...
    contents: List[Label] = ...
    def process(self) -> None: ...

class FontFunction(OneParamFunction):
    commandmap: Dict[str, str] = ...
    type: str = ...
    def process(self) -> None: ...

class BigSymbol:
    symbols: Dict[str, List[str]] = ...
    symbol: str = ...
    def __init__(self, symbol: str) -> None: ...
    def getpieces(self) -> List[str]: ...
    def smalllimit(self) -> bool: ...

class BigBracket(BigSymbol):
    size: int = ...
    original: str = ...
    alignment: str = ...
    pieces: Optional[List[str]] = ...
    def __init__(self, size: int, bracket: str, alignment: str = ...) -> None: ...
    def getpiece(self, index): ...
    def getpiece1(self, index: int) -> str: ...
    def getpiece3(self, index: int) -> str: ...
    def getpiece4(self, index: int) -> str: ...
    def getcell(self, index: int) -> TaggedBit: ...
    def getcontents(self) -> List[TaggedBit]: ...
    def getsinglebracket(self) -> List[TaggedBit]: ...

class FormulaEquation(CommandBit):
    piece: str = ...
    output: ContentsOutput = ...  # type: ignore
    def parsebit(self, pos: Position) -> None: ...

class FormulaCell(FormulaCommand):
    alignment: str = ...
    output: TaggedOutput = ...
    def setalignment(self, alignment: str) -> math2html.FormulaCell: ...
    def parsebit(self, pos: Position) -> None: ...

class FormulaRow(FormulaCommand):
    cellseparator: str = ...
    alignments: str = ...
    output: TaggedOutput = ...
    def setalignments(self, alignments: str) -> math2html.FormulaRow: ...
    def parsebit(self, pos: Position) -> None: ...
    def createcell(self, index) -> FormulaCell: ...

class MultiRowFormula(CommandBit):
    rows: List = ...
    size: int = ...
    def parserows(self, pos: Position) -> None: ...
    def iteraterows(self, pos: Position) -> Iterator[FormulaRow]: ...
    def addempty(self) -> None: ...
    def addrow(self, row: FormulaRow) -> None: ...

class FormulaArray(MultiRowFormula):
    piece: str = ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...
    valign: str = ...
    alignments: List[str] = ...
    def parsealignments(self, pos: Position) -> None: ...

class FormulaMatrix(MultiRowFormula):
    piece: str = ...
    output: TaggedOutput = ...
    valign: str = ...
    alignments: List[str] = ...
    def parsebit(self, pos: Position) -> None: ...

class FormulaCases(MultiRowFormula):
    piece: str = ...
    output: ContentsOutput = ...  # type: ignore
    alignments: List[str] = ...
    contents: List[TaggedBit] = ...
    def parsebit(self, pos: Position) -> None: ...

class EquationEnvironment(MultiRowFormula):
    output: TaggedOutput = ...
    alignments: List[str] = ...
    def parsebit(self, pos: Position) -> None: ...

class BeginCommand(CommandBit):
    commandmap: Dict[str, str] = ...
    types: List[Type] = ...
    size: int = ...
    def parsebit(self, pos: Position) -> None: ...
    def findbit(self, piece: str) -> EquationEnvironment: ...

class CombiningFunction(OneParamFunction):
    commandmap: Dict[str, str] = ...
    type: str = ...
    def parsebit(self, pos: Position) -> None: ...
    def parsesingleparameter(self, pos: Position) -> FormulaConstant: ...

class DecoratingFunction(OneParamFunction):
    commandmap: Dict[str, str] = ...
    type: str = ...
    symbol: TaggedBit = ...
    parameter: FormulaConstant = ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...

class LimitCommand(EmptyCommand):
    commandmap: Dict[str, str] = ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...

class LimitPreviousCommand(LimitCommand):
    commandmap: Any = ...
    output: TaggedOutput = ...
    def parsebit(self, pos: Position) -> None: ...
    def __unicode__(self) -> str: ...

class LimitsProcessor(MathsProcessor):
    def process(self, contents: List[Any], index: int) -> None: ...
    def checklimits(self, contents: List[Any], index: int) -> bool: ...
    def limitsahead(self, contents: List[Any], index: int) -> None: ...
    def modifylimits(self, contents: List[Any], index: int) -> None: ...
    def getlimit(self, contents: List[Any], index: int) -> Any: ...
    def modifyscripts(self, contents: List[Any], index: int) -> None: ...
    def checkscript(self, contents: List[Any], index: int) -> bool: ...
    def checkcommand(self, contents: List[Any], index: int, type: Type) -> bool: ...
    def getscript(self, contents: List[Any], index: int) -> Any: ...

class BracketCommand(OneParamFunction):
    commandmap: Dict[str, str] = ...
    def parsebit(self, pos: Position) -> None: ...
    original: str = ...
    command: str = ...
    contents: List[FormulaConstant] = ...
    def create(self, direction: str, character: str) -> math2html.BracketCommand: ...

class BracketProcessor(MathsProcessor):
    def process(self, contents: List[Any], index: int) -> None: ...
    def processleft(self, contents: List[Any], index: int) -> None: ...
    def checkleft(self, contents: List[Any], index: int) -> bool: ...
    def checkright(self, contents: List[Any], index: int) -> bool: ...
    def checkdirection(self, bit: BracketCommand, command: str) -> bool: ...
    def findright(self, contents: List[Any], index: int) -> Optional[int]: ...
    def findmax(self, contents: List[Any], leftindex: int, rightindex: int) -> int: ...
    def resize(self, command: BracketCommand, size: int) -> None: ...

class TodayCommand(EmptyCommand):
    commandmap: Any = ...
    output: FixedOutput = ...  # type: ignore
    html: List[str] = ...
    def parsebit(self, pos: Position) -> None: ...

class ParameterDefinition:
    parambrackets: List[Tuple[str, str]] = ...
    name: Optional[str] = ...
    literal: bool = ...
    optional: bool = ...
    value: Any = ...
    literalvalue: Any = ...
    def __init__(self) -> None: ...
    def parse(self, pos: Position) -> Optional[math2html.ParameterDefinition]: ...
    def read(self, pos: Position, function: CommandBit) -> None: ...
    def __unicode__(self) -> str: ...

class ParameterFunction(CommandBit):
    params: Dict[str, math2html.ParameterFunction] = ...
    def readparams(self, readtemplate: str, pos: Position) -> None: ...
    def paramdefs(self, readtemplate: str) -> Iterator[ParameterDefinition]: ...
    def getparam(self, name: str) -> ParameterDefinition: ...
    def getvalue(self, name: str) -> ParameterDefinition: ...
    def getliteralvalue(self, name: str) -> Any: ...

class HybridFunction(ParameterFunction):
    commandmap: Dict[str, List[str]] = ...
    contents: Any = ...
    def parsebit(self, pos: Position) -> None: ...
    def writeparams(self, writetemplate) -> List[Any]: ...
    def writepos(self, pos: Position) -> List[Any]: ...
    def writeparam(self, pos: Position) -> Any: ...
    def writefunction(self, pos: Position) -> Optional[TaggedBit]: ...
    def readtag(self, pos: Position) -> Optional[str]: ...
    def writebracket(self, direction: str, character: str) -> BracketCommand: ...
    size: int = ...
    def computehybridsize(self) -> None: ...

class HybridSize:
    configsizes: Dict[str, str] = ...
    def getsize(self, function: ParameterFunction) -> int: ...

class HeaderParser(Parser):
    def parse(self, reader: LineReader) -> List[Any]: ...
    def parseline(self, reader: LineReader, contents: List[Any]) -> None: ...
    def parsebranch(self, reader: LineReader) -> None: ...
    ending: str = ...
    def complete(self, ending: str) -> math2html.HeaderParser: ...

class PreambleParser(Parser):
    preamble: ClassVar[List[str]] = ...
    ending: str = ...
    def parse(self, reader: LineReader) -> List: ...
    def parsepreambleline(self, reader: LineReader) -> None: ...

class LstParser:
    globalparams: Dict[str, str] = ...
    def parselstset(self, reader: LineReader) -> None: ...
    def extractlstset(self, reader: LineReader) -> Optional[str]: ...
    def parsecontainer(self, container: Container) -> None: ...
    def parselstparams(self, paramlist: Sequence[str]) -> Dict[str, str]: ...

class MacroDefinition(CommandBit):
    macros: Dict[str, math2html.MacroDefinition] = ...
    output: EmptyOutput = ...  # type: ignore
    parameternumber: int = ...
    defaults: List[Any] = ...
    def parsebit(self, pos: Position) -> None: ...
    newcommand: str = ...
    definition: FormulaConstant = ...
    def parseparameters(self, pos: Position) -> None: ...
    def parsenewcommand(self, pos: Position) -> Union[str]: ...
    def instantiate(self) -> FormulaConstant: ...

class MacroParameter(FormulaBit):
    def detect(self, pos: Position) -> bool: ...
    number: int = ...
    original: str = ...
    contents: List[TaggedBit] = ...
    def parsebit(self, pos: Position) -> None: ...

class MacroFunction(CommandBit):
    commandmap: Any = ...
    output: FilteredOutput = ...  # type: ignore
    values: List[Any] = ...
    def parsebit(self, pos: Position) -> None: ...
    def parseparameters(self, pos: Position, macro: MacroDefinition) -> None: ...
    def parseoptional(self, pos: Position, defaults: List[Any]) -> None: ...
    def parsemandatory(self, pos: Position, number: int) -> None: ...
    def parsemacroparameter(self, pos: Position, remaining: int) -> Optional[Union[FormulaNumber, FormulaConstant]]: ...
    def parsenumbers(self, pos: Position, remaining: int) -> Optional[FormulaNumber]: ...
    contents: List[FormulaConstant] = ...
    def completemacro(self, macro: MacroDefinition): ...
    def addfilter(self, index: int, value: str): ...

class FormulaMacro(Formula):
    parser: MacroParser = ...
    output: EmptyOutput = ...  # type: ignore
    def __init__(self) -> None: ...
    def __unicode__(self) -> str: ...

def math2html(formula: str) -> str: ...
def main() -> None: ...



# Stubs for docutils.frontend (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import configparser as CP
from docutils import frontend
from docutils.utils import DependencyList
import docutils.nodes
import optparse
from typing import Any, Dict, List, MutableSequence, Optional, Sequence, Tuple, Union

__docformat__: str

def store_multiple(option, opt, value: Any, parser, *args, **kwargs) -> None: ...
def read_config_file(option, opt, value, parser) -> None: ...
def validate_encoding(setting: str, value: str, option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> str: ...
def validate_encoding_error_handler(setting, value: str, option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> str: ...
def validate_encoding_and_error_handler(setting: str, value: str, option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> str: ...
def validate_boolean(setting: str, value: Union[bool, str], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> bool: ...
def validate_ternary(setting: str, value: Union[bool, str], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> Optional[bool]: ...
def validate_nonnegative_int(setting: str, value: Union[float, int, str], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> int: ...
def validate_threshold(setting: str, value: Any, option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> int: ...
def validate_colon_separated_string_list(setting: str, value: Union[str, List[str]], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> Union[str, List[str]]: ...
def validate_comma_separated_list(setting: str, value: Union[str, List[str]], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> List[str]: ...
def validate_url_trailing_slash(setting: str, value: str, option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> str: ...
def validate_dependency_file(setting: str, value: Any, option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> DependencyList: ...
def validate_strip_class(setting: str, value: List[str], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> List[str]: ...
def validate_smartquotes_locales(setting: str, value: Union[str, List[str]], option_parser: optparse.OptionParser, config_parser: Optional[Any] = ..., config_section: Optional[Any] = ...) -> List[Tuple[str, str]]: ...
def make_paths_absolute(pathdict: Dict[Any, Union[str, List[str]]], keys: Sequence[Any], base_path: Optional[str] = ...) -> None: ...
def make_one_path_absolute(base_path: str, path: str) -> str: ...
def filter_settings_spec(settings_spec: Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]], *exclude, **replace) -> Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]]: ...

class Values(optparse.Values):
    record_dependencies: DependencyList = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def update(self, other_dict, option_parser: optparse.OptionParser) -> None: ...
    def copy(self) -> frontend.Values: ...

class Option(optparse.Option):
    ATTRS: List[str] = ...
    def process(self, opt: str, value, values, parser) -> int: ...

class OptionParser(optparse.OptionParser, docutils.SettingsSpec):
    standard_config_files: List[str] = ...
    threshold_choices: List[str] = ...
    thresholds: Dict[str, int] = ...
    booleans: Dict[str, bool] = ...
    default_error_encoding: Optional[str] = ...
    default_error_encoding_error_handler: str = ...
    settings_spec: Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]] = ...
    settings_defaults: Dict[str, Any] = ...
    relative_path_settings: List[str] = ...
    config_section: str = ...
    version_template: str = ...
    lists: Dict = ...
    config_files: List[str] = ...
    version: str = ...
    components: Any = ...
    def __init__(self, components: Any = ..., defaults: Optional[Any] = ..., read_config_files: Optional[Any] = ..., *args, **kwargs) -> None: ...
    def populate_from_components(self, components: Sequence[Any]) -> None: ...
    def get_standard_config_files(self) -> List[str]: ...
    def get_standard_config_settings(self) -> Values: ...
    def get_config_file_settings(self, config_file: str) -> Dict[str, Any]: ...
    def check_values(self, values, args: MutableSequence[str]): ...
    def check_args(self, args: MutableSequence[str]) -> Tuple[Optional[str], Optional[str]]: ...
    def set_defaults_from_dict(self, defaults: Dict[str, Any]) -> None: ...
    def get_default_values(self) -> Values: ...
    def get_option_by_dest(self, dest) -> Option: ...

class ConfigParser(CP.RawConfigParser):
    old_settings: Dict[str, Tuple[str, str]] = ...
    old_warning: str = ...
    not_utf8_error: str = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def read(self, filenames: Union[str, Sequence[str]], option_parser: OptionParser): ...  # type: ignore
    def handle_old_config(self, filename: str): ...
    def validate_settings(self, filename: str, option_parser: OptionParser) -> None: ...
    def optionxform(self, optionstr: str) -> str: ...
    def get_section(self, section: str): ...

class ConfigDeprecationWarning(DeprecationWarning): ...



# Stubs for docutils.transforms (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import ApplicationError, Component, nodes, TransformSpec
from docutils.nodes import Node, pending
from types import ModuleType
from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Type

__docformat__: str

class TransformError(ApplicationError): ...

class Transform:
    default_priority: Optional[int] = ...
    document: nodes.document = ...
    startnode: Optional[Node] = ...
    language: ModuleType = ...
    def __init__(self, document: nodes.document, startnode: Optional[Node] = ...) -> None: ...
    def apply(self, **kwargs: Any) -> None: ...

class Transformer(TransformSpec):
    transforms: List[Tuple[str, Type[Transform], Optional[Node]]] = ...
    unknown_reference_resolvers: List[Callable[[Node], bool]] = ...
    document: nodes.document = ...
    applied: List[Tuple[int, Type[Transform], pending, Any]] = ...
    sorted: int = ...
    components: Dict[str, Component] = ...
    serialno: int = ...
    def __init__(self, document: nodes.document) -> None: ...
    def add_transform(self, transform_class: Type[Transform], priority: Optional[int] = ..., **kwargs) -> None: ...
    def add_transforms(self, transform_list: List[Type[Transform]]) -> None: ...
    def add_pending(self, pending: pending, priority: Optional[int] = ...) -> None: ...
    def get_priority_string(self, priority: int) -> str: ...
    def populate_from_components(self, components: Iterable[Component]) -> None: ...
    def apply_transforms(self) -> None: ...



# Stubs for docutils.parsers.rst.tableparser (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import DataError
from docutils.statemachine import StringList
from typing import Dict, List, Optional, Pattern, Tuple

Cell = Tuple[int, int, int, List[str]]
Row = List[Optional[Cell]]
Colspecs = List[int]

__docformat__: str

class TableMarkupError(DataError):
    offset: int = ...
    def __init__(self, *args, **kwargs) -> None: ...

class TableParser:
    head_body_separator_pat: Pattern = ...
    double_width_pad_char: str = ...
    def parse(self, block: StringList) -> Tuple[Colspecs, List[Row], List[Row]]: ...
    head_body_sep: int = ...
    def find_head_body_sep(self) -> None: ...

class GridTableParser(TableParser):
    head_body_separator_pat: Pattern = ...
    block: StringList = ...
    bottom: int = ...
    right: int = ...
    head_body_sep: int = ...
    done: List[int] = ...
    cells: List[Cell] = ...
    rowseps: Dict[int, List[int]] = ...
    colseps: Dict[int, List[int]] = ...
    def setup(self, block: StringList) -> None: ...
    def parse_table(self) -> None: ...
    def mark_done(self, top: int, left: int, bottom: int, right: int) -> None: ...
    def check_parse_complete(self) -> bool: ...
    def scan_cell(self, top: int, left: int) -> Tuple[int, int, Dict[int, List[int]], Dict[int, List[int]]]: ...
    def scan_right(self, top: int, left: int) -> Tuple[int, int, Dict[int, List[int]], Dict[int, List[int]]]: ...
    def scan_down(self, top: int, left: int, right: int) -> Tuple[int, Dict[int, List[int]], Dict[int, List[int]]]: ...
    def scan_left(self, top: int, left: int, bottom: int, right: int) -> Tuple[Dict[int, List[int]], Dict[int, List[int]]]: ...
    def scan_up(self, top: int, left: int, bottom: int, right: int) -> Dict[int, List[int]]: ...
    def structure_from_cells(self) -> Tuple[Colspecs, List[Row], List[Row]]: ...

class SimpleTableParser(TableParser):
    head_body_separator_pat: Pattern = ...
    span_pat: Pattern = ...
    block: StringList = ...
    head_body_sep: int = ...
    columns: List[Tuple[int, int]] = ...
    border_end: int = ...
    table: Tuple[List[int], List[Row], List[Row]] = ...
    done: List[int] = ...
    rowseps: Dict[int, Tuple[int]] = ...
    colseps: Dict[int, Tuple[int]] = ...
    def setup(self, block: StringList) -> None: ...
    def parse_table(self) -> None: ...
    def parse_columns(self, line: str, offset: int) -> List[Tuple[int, int]]: ...
    def init_row(self, colspec: List[Tuple[int, int]], offset: int) -> List[Cell]: ...
    def parse_row(self, lines: List[str], start: int, spanline: Optional[Tuple[str, int]] = ...) -> None: ...
    def check_columns(self, lines: List[str], first_line: int, columns: List[Tuple[int, int]]): ...
    def structure_from_cells(self) -> Tuple[Colspecs, List[Row], List[Row]]: ...

def update_dict_of_lists(master: Dict[int, List[int]], newdata: Dict[int, List[int]]) -> None: ...



# Stubs for docutils.transforms.parts (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes
from docutils.nodes import bullet_list, Node
from docutils.transforms import Transform
from typing import Callable, List, Optional, Tuple, TypeVar, Union

N_co = TypeVar('N_co', bound=Node, covariant=True)

__docformat__: str

class SectNum(Transform):
    default_priority: int = ...
    maxdepth: Optional[int] = ...
    startvalue: int = ...
    prefix: str = ...
    suffix: str = ...
    def apply(self) -> None: ...  # type: ignore
    def update_section_numbers(self, node: Node, prefix: Tuple[str] = ..., depth: int = ...) -> None: ...

class Contents(Transform):
    default_priority: int = ...
    toc_id: str = ...
    backlinks: str = ...
    def apply(self) -> None: ...  # type: ignore
    def build_contents(self, node: Node, level: int = ...) -> Union[bullet_list, list]: ...
    def copy_and_filter(self, node: Node) -> List[N_co]: ...

class ContentsFilter(nodes.TreeCopyVisitor):
    def get_entry_text(self) -> List[N_co]: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def ignore_node_but_process_children(self, node: nodes.Element) -> None: ...
    visit_interpreted: Callable[[nodes.Element], None] = ...
    visit_problematic: Callable[[nodes.Element], None] = ...
    visit_reference: Callable[[nodes.Element], None] = ...
    visit_target: Callable[[nodes.Element], None] = ...



# Stubs for docutils.utils.smartquotes (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Dict, Generator, Iterable, Tuple

options: str
default_smartypants_attr: str

class smartchars:
    endash: str = ...
    emdash: str = ...
    ellipsis: str = ...
    apostrophe: str = ...
    quotes: Dict[str, Any] = ...
    language: str = ...
    def __init__(self, language: str = ...) -> None: ...

def smartyPants(text: str, attr: str = ..., language: str = ...) -> str: ...
def educate_tokens(text_tokens: Iterable[Tuple[str, str]], attr: str = ..., language: str = ...) -> Generator[str, None, None]: ...
def educateQuotes(text: str, language: str = ...) -> str: ...
def educateBackticks(text: str, language: str = ...) -> str: ...
def educateSingleBackticks(text: str, language: str = ...) -> str: ...
def educateDashes(text: str) -> str: ...
def educateDashesOldSchool(text: str) -> str: ...
def educateDashesOldSchoolInverted(text: str) -> str: ...
def educateEllipses(text: str) -> str: ...
def stupefyEntities(text: str, language: str = ...) -> str: ...
def processEscapes(text: str, restore: bool = ...) -> str: ...
def tokenize(text: str) -> Generator[Tuple[str, str], None, None]: ...



# Stubs for docutils.utils.error_reporting (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import IO, Optional

locale_encoding: str

class SafeString:
    data: str = ...
    encoding: str = ...
    encoding_errors: str = ...
    decoding_errors: str = ...
    def __init__(self, data: str, encoding: Optional[str] = ..., encoding_errors: str = ..., decoding_errors: str = ...) -> None: ...
    def __unicode__(self) -> str: ...

class ErrorString(SafeString):
    def __unicode__(self) -> str: ...

class ErrorOutput:
    stream: IO = ...
    encoding: str = ...
    encoding_errors: str = ...
    decoding_errors: str = ...
    def __init__(self, stream: Optional[IO] = ..., encoding: Optional[str] = ..., encoding_errors: str = ..., decoding_errors: str = ...) -> None: ...
    def write(self, data: str) -> None: ...
    def close(self) -> None: ...



# Stubs for docutils.writers.manpage (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes, SettingsSpec, writers
from docutils.parsers.rst.directives.html import MetaBody
from docutils.writers import manpage
from types import ModuleType
from typing import Any, Callable, Dict, List, Optional, Pattern, Tuple, Type

__docformat__: str
FIELD_LIST_INDENT: int
DEFINITION_LIST_INDENT: int
OPTION_LIST_INDENT: int
BLOCKQOUTE_INDENT: float
LITERAL_BLOCK_INDENT: float
MACRO_DEF: str

class Writer(writers.Writer):
    supported: Tuple[str, ...] = ...
    output: Any = ...
    translator_class: Type[manpage.Translator] = ...
    def __init__(self) -> None: ...
    def translate(self) -> None: ...

class Table:
    def __init__(self) -> None: ...
    def new_row(self) -> None: ...
    def append_separator(self, separator: str) -> None: ...
    def append_cell(self, cell_lines: List[str]) -> None: ...
    def as_list(self) -> List[str]: ...

class Translator(nodes.NodeVisitor):
    words_and_spaces: Pattern[str] = ...
    possibly_a_roff_command: Pattern[str] = ...
    document_start: str = ...
    settings: SettingsSpec = ...
    language: ModuleType = ...
    head: List[str] = ...
    body: List[str] = ...
    foot: List[str] = ...
    section_level: int = ...
    context: List[Any] = ...
    topic_class: str = ...
    colspecs: List[nodes.Node] = ...
    compact_p: int = ...
    compact_simple: Any = ...
    header_written: int = ...
    authors: List[Any] = ...
    defs: Dict[str, Tuple[str, str]] = ...
    def __init__(self, document: nodes.document) -> None: ...
    def comment_begin(self, text: str) -> str: ...
    def comment(self, text: str) -> str: ...
    def ensure_eol(self) -> None: ...
    def astext(self) -> str: ...
    def deunicode(self, text: str) -> str: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def list_start(self, node: nodes.Node) -> None: ...
    def list_end(self) -> None: ...
    def header(self) -> str: ...
    def append_header(self) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_admonition(self, node: nodes.Node, name: Optional[str] = ...) -> None: ...
    def depart_admonition(self, node: nodes.admonition) -> None: ...
    def visit_attention(self, node: nodes.attention) -> None: ...
    depart_attention: Callable[[nodes.Node], None] = ...
    def visit_docinfo_item(self, node: nodes.Node, name: str) -> None: ...
    def depart_docinfo_item(self, node: nodes.Node) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    depart_author: Callable[[nodes.Node], None] = ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_block_quote(self, node: nodes.block_quote) -> None: ...
    def depart_block_quote(self, node: nodes.block_quote) -> None: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def depart_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_caption(self, node: nodes.caption) -> None: ...
    def depart_caption(self, node: nodes.caption) -> None: ...
    def visit_caution(self, node: nodes.caution) -> None: ...
    depart_caution: Callable[[nodes.Node], None] = ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    def write_colspecs(self) -> None: ...
    def visit_comment(self, node: nodes.comment, sub: Callable[[str, str], str] = ...) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    depart_contact: Callable[[nodes.Node], None] = ...
    def visit_container(self, node: nodes.container) -> None: ...
    def depart_container(self, node: nodes.container) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def visit_danger(self, node: nodes.danger) -> None: ...
    depart_danger: Callable[[nodes.Node], None] = ...
    def visit_date(self, node: nodes.date) -> None: ...
    def visit_decoration(self, node: nodes.decoration) -> None: ...
    def depart_decoration(self, node: nodes.decoration) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def depart_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def visit_document(self, node: nodes.document) -> None: ...
    def depart_document(self, node: nodes.document) -> None: ...
    def visit_emphasis(self, node: nodes.emphasis) -> None: ...
    def depart_emphasis(self, node: nodes.emphasis) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    def depart_entry(self, node: nodes.entry) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_error(self, node: nodes.error) -> None: ...
    depart_error: Callable[[nodes.Node], None] = ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_figure(self, node: nodes.figure) -> None: ...
    def depart_figure(self, node: nodes.figure) -> None: ...
    def visit_footer(self, node: nodes.footer) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    def footnote_backrefs(self, node: nodes.Node) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def depart_generated(self, node: nodes.generated) -> None: ...
    def visit_header(self, node: nodes.header) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def visit_hint(self, node: nodes.hint) -> None: ...
    depart_hint: Callable[[nodes.Node], None] = ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_attribution(self, node: nodes.attribution) -> None: ...
    def depart_attribution(self, node: nodes.attribution) -> None: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def visit_important(self, node: nodes.important) -> None: ...
    depart_important: Callable[[nodes.Node], None] = ...
    def visit_inline(self, node: nodes.inline) -> None: ...
    def depart_inline(self, node: nodes.inline) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_legend(self, node: nodes.legend) -> None: ...
    def depart_legend(self, node: nodes.legend) -> None: ...
    def visit_line_block(self, node: nodes.line_block) -> None: ...
    def depart_line_block(self, node: nodes.line_block) -> None: ...
    def visit_line(self, node: nodes.line) -> None: ...
    def depart_line(self, node: nodes.line) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    def visit_math(self, node: nodes.math) -> None: ...
    def depart_math(self, node: nodes.math) -> None: ...
    def visit_math_block(self, node: nodes.math_block) -> None: ...
    def depart_math_block(self, node: nodes.math_block) -> None: ...
    def visit_meta(self, node: MetaBody.meta) -> None: ...
    def depart_meta(self, node: MetaBody.meta) -> None: ...
    def visit_note(self, node: nodes.note) -> None: ...
    depart_note: Callable[[nodes.Node], None] = ...
    def indent(self, by: float = ...) -> None: ...
    def dedent(self) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option(self, node: nodes.option) -> None: ...
    def depart_option(self, node: nodes.option) -> None: ...
    def visit_option_string(self, node: nodes.option_string) -> None: ...
    def depart_option_string(self, node: nodes.option_string) -> None: ...
    def visit_option_argument(self, node: nodes.option_argument) -> None: ...
    def depart_option_argument(self, node: nodes.option_argument) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def depart_organization(self, node: nodes.organization) -> None: ...
    def first_child(self, node: nodes.Node) -> int: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_problematic(self, node: nodes.problematic) -> None: ...
    def depart_problematic(self, node: nodes.problematic) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def depart_reference(self, node: nodes.reference) -> None: ...
    def visit_revision(self, node: nodes.revision) -> None: ...
    depart_revision: Callable[[nodes.Node], None] = ...
    def visit_row(self, node: nodes.row) -> None: ...
    def depart_row(self, node: nodes.row) -> None: ...
    def visit_section(self, node: nodes.section) -> None: ...
    def depart_section(self, node: nodes.section) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    depart_status: Callable[[nodes.Node], None] = ...
    def visit_strong(self, node: nodes.strong) -> None: ...
    def depart_strong(self, node: nodes.strong) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_substitution_reference(self, node: nodes.substitution_reference) -> None: ...
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def depart_tgroup(self, node: nodes.tgroup) -> None: ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...
    def visit_tip(self, node: nodes.tip) -> None: ...
    depart_tip: Callable[[nodes.Node], None] = ...
    def visit_title(self, node: nodes.title) -> None: ...
    def depart_title(self, node: nodes.title) -> None: ...
    def visit_title_reference(self, node: nodes.title_reference) -> None: ...
    def depart_title_reference(self, node: nodes.title_reference) -> None: ...
    def visit_topic(self, node: nodes.topic) -> None: ...
    def depart_topic(self, node: nodes.topic) -> None: ...
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def depart_sidebar(self, node: nodes.sidebar) -> None: ...
    def visit_rubric(self, node: nodes.rubric) -> None: ...
    def depart_rubric(self, node: nodes.rubric) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...
    def depart_transition(self, node: nodes.transition) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def visit_warning(self, node: nodes.warning) -> None: ...
    depart_warning: Callable[[nodes.Node], None] = ...
    def unimplemented_visit(self, node: nodes.Node) -> None: ...



# Stubs for docutils.statemachine (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Callable, Dict, Generator, Iterator, List, Match, Optional, overload, Pattern, Sequence, Tuple, Type, Union
from docutils import statemachine

__docformat__: str

class StateMachine:
    input_lines: statemachine.StringList = ...
    input_offset: int = ...
    line: str = ...
    line_offset: int = ...
    debug: bool = ...
    initial_state: str = ...
    current_state: List[statemachine.State] = ...
    states: Any = ...
    observers: Any = ...
    def __init__(self, state_classes: Sequence[Type[statemachine.State]], initial_state: str, debug: bool = ...) -> None: ...
    def unlink(self) -> None: ...
    def run(self, input_lines: Union[List[str], statemachine.StringList], input_offset: int = ..., context: Optional[Any] = ..., input_source: Optional[str] = ..., initial_state: Optional[str] = ...) -> List[Any]: ...
    def get_state(self, next_state: Optional[str] = ...) -> statemachine.State: ...
    def next_line(self, n: int = ...) -> str: ...
    def is_next_line_blank(self) -> bool: ...
    def at_eof(self) -> bool: ...
    def at_bof(self) -> bool: ...
    def previous_line(self, n: int = ...) -> str: ...
    def goto_line(self, line_offset) -> None: ...
    def get_source(self, line_offset) -> str: ...
    def abs_line_offset(self) -> int: ...
    def abs_line_number(self) -> int: ...
    def get_source_and_line(self, lineno: Optional[int] = ...) -> Tuple[str, int]: ...
    def insert_input(self, input_lines: Union[List[str], statemachine.StringList], source: str) -> None: ...
    def get_text_block(self, flush_left: bool = ...) -> statemachine.StringList: ...
    def check_line(self, context: Any, state: statemachine.State, transitions: Optional[List[str]] = ...) -> Any: ...
    def add_state(self, state_class: Type[statemachine.State]) -> None: ...
    def add_states(self, state_classes: Sequence[Type[statemachine.State]]) -> None: ...
    def runtime_init(self) -> None: ...
    def error(self) -> None: ...
    def attach_observer(self, observer) -> None: ...
    def detach_observer(self, observer) -> None: ...
    def notify_observers(self) -> None: ...

class State:
    patterns: Dict[str, Union[str, Pattern]] = ...
    initial_transitions: Union[Sequence[str], Sequence[Tuple[str, str]]] = ...
    nested_sm: Type[StateMachine] = ...
    nested_sm_kwargs: Dict[str, Any] = ...
    transition_order: List[str] = ...
    transitions: Dict[str, Tuple[Pattern, Callable, str]] = ...
    state_machine: StateMachine = ...
    debug: bool = ...
    def __init__(self, state_machine: StateMachine, debug: bool = ...) -> None: ...
    def runtime_init(self) -> None: ...
    def unlink(self) -> None: ...
    def add_initial_transitions(self) -> None: ...
    def add_transitions(self, names: List[str], transitions) -> None: ...
    def add_transition(self, name: str, transition: Tuple[Pattern, str, str]) -> None: ...
    def remove_transition(self, name: str) -> None: ...
    def make_transition(self, name: str, next_state: Optional[str] = ...) -> Tuple[Pattern, Callable, str]: ...
    def make_transitions(self, name_list: List[Union[str, Tuple[str], Tuple[str, str]]]) -> Tuple[List[str], Dict[str, Tuple[Pattern, Callable, str]]]: ...
    def no_match(self, context, transitions: Tuple[List[str], Dict[str, Tuple[Pattern, Callable, str]]]) -> Tuple[Any, str, List]: ...
    def bof(self, context) -> Tuple[Any, List]: ...
    def eof(self, context) -> List: ...
    def nop(self, match, context, next_state: str) -> Tuple[Any, str, List]: ...

class StateMachineWS(StateMachine):
    def get_indented(self, until_blank: bool = ..., strip_indent: bool = ...) -> Tuple[statemachine.StringList, int, int, bool]: ...
    def get_known_indented(self, indent: int, until_blank: bool = ..., strip_indent: bool = ...) -> Tuple[List[str], int, bool]: ...
    def get_first_known_indented(self, indent: int, until_blank: bool = ..., strip_indent: bool = ..., strip_top: bool = ...) -> Tuple[List[str], int, int, bool]: ...

class StateWS(State):
    indent_sm: Type[StateMachine] = ...
    indent_sm_kwargs: Dict[str, Any] = ...
    known_indent_sm: Type[StateMachine] = ...
    known_indent_sm_kwargs: Dict[str, Any] = ...
    ws_patterns: Dict[str, str] = ...
    ws_initial_transitions: Sequence[str] = ...
    def __init__(self, state_machine: StateMachine, debug: bool = ...) -> None: ...
    patterns: Dict[str, Union[str, Pattern]] = ...
    def add_initial_transitions(self) -> None: ...
    def blank(self, match: Match, context: Any, next_state: str) -> Tuple[Any, str, Any]: ...
    def indent(self, match: Match, context: Any, next_state: str) -> Optional[Tuple[Any, str, Any]]: ...
    def known_indent(self, match: Match, context: Any, next_state: str) -> Tuple[Any, str, Any]: ...
    def first_known_indent(self, match: Match, context: Any, next_state: str) -> Tuple[Any, str, Any]: ...

class _SearchOverride:
    def match(self, pattern: Pattern) -> Match: ...

class SearchStateMachine(_SearchOverride, StateMachine): ...
class SearchStateMachineWS(_SearchOverride, StateMachineWS): ...

class ViewList:
    data: List = ...
    items: List[Tuple[str, int]] = ...
    parent: statemachine.ViewList = ...
    parent_offset: int = ...
    def __init__(self, initlist: Optional[Union[statemachine.ViewList, List]] = ..., source: Optional[str] = ..., items: Optional[List[Tuple[str, int]]] = ..., parent: Optional[statemachine.ViewList] = ..., parent_offset: Optional[int] = ...) -> None: ...
    def __lt__(self, other: Any) -> bool: ...
    def __le__(self, other: Any) -> bool: ...
    def __eq__(self, other: Any) -> bool: ...
    def __ne__(self, other: Any) -> bool: ...
    def __gt__(self, other: Any) -> bool: ...
    def __ge__(self, other: Any) -> bool: ...
    def __cmp__(self, other: Any) -> int: ...
    def __contains__(self, item: statemachine.ViewList) -> bool: ...
    def __len__(self) -> int: ...

    @overload
    def __getitem__(self, i: slice) -> statemachine.ViewList: ...
    @overload  # NOQA: F811
    def __getitem__(self, i: int) -> Any: ...

    @overload
    def __setitem__(self, i: slice, item: statemachine.ViewList) -> None: ...
    @overload  # NOQA: F811
    def __setitem__(self, i: int, item: Any) -> None: ...

    def __delitem__(self, i: int) -> None: ...
    def __add__(self, other: statemachine.ViewList) -> None: ...
    def __radd__(self, other: statemachine.ViewList) -> None: ...
    def __iadd__(self, other: statemachine.ViewList) -> statemachine.ViewList: ...
    def __mul__(self, n: int) -> statemachine.ViewList: ...
    __rmul__: Any = ...
    def __imul__(self, n: int) -> statemachine.ViewList: ...
    def extend(self, other: statemachine.ViewList) -> None: ...
    def append(self, item: Any, source: Optional[str] = ..., offset: int = ...) -> None: ...
    def insert(self, i: int, item: Any, source: Optional[str] = ..., offset: int = ...) -> None: ...
    def pop(self, i: int = ...) -> Any: ...
    def trim_start(self, n: int = ...) -> None: ...
    def trim_end(self, n: int = ...) -> None: ...
    def remove(self, item: Any) -> None: ...
    def count(self, item: Any) -> int: ...
    def index(self, item: Any) -> int: ...
    def reverse(self) -> None: ...
    def sort(self, *args: Any) -> None: ...
    def info(self, i: int) -> Tuple[str, int]: ...
    def source(self, i: int) -> str: ...
    def offset(self, i: int) -> int: ...
    def disconnect(self) -> None: ...
    def xitems(self) -> Generator[Tuple[str, int, str], None, None]: ...
    def pprint(self) -> None: ...

    # dummy atribute to indicate to mypy that ViewList is Iterable
    def __iter__(self) -> Iterator[Any]: ...

class StringList(ViewList):
    def trim_left(self, length: int, start: int = ..., end: int = ...) -> None: ...
    def get_text_block(self, start: int, flush_left: bool = ...) -> statemachine.StringList: ...
    def get_indented(self, start: int = ..., until_blank: bool = ..., strip_indent: bool = ..., block_indent: Optional[int] = ..., first_indent: Optional[int] = ...) -> Tuple[statemachine.StringList, int, bool]: ...
    def get_2D_block(self, top: int, left: int, bottom: int, right: int, strip_indent: bool = ...) -> statemachine.StringList: ...
    def pad_double_width(self, pad_char: str) -> None: ...
    def replace(self, old: str, new: str) -> None: ...

    # dummy atribute to indicate to mypy that ViewList is Iterable[str]
    def __iter__(self) -> Iterator[str]: ...

class StateMachineError(Exception): ...
class UnknownStateError(StateMachineError): ...
class DuplicateStateError(StateMachineError): ...
class UnknownTransitionError(StateMachineError): ...
class DuplicateTransitionError(StateMachineError): ...
class TransitionPatternNotFound(StateMachineError): ...
class TransitionMethodNotFound(StateMachineError): ...
class UnexpectedIndentationError(StateMachineError): ...
class TransitionCorrection(Exception): ...
class StateCorrection(Exception): ...

def string2lines(astring: str, tab_width: int = ..., convert_whitespace: bool = ..., whitespace: Pattern = ...) -> List[str]: ...



# Stubs for docutils.writers.docutils_xml (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import docutils
from docutils import nodes, writers
from docutils.writers import docutils_xml
from typing import Any, Dict, List, Sequence, Tuple
import xml.sax.saxutils
import xml.sax.xmlreader.XMLReader

__docformat__: str

class RawXmlError(docutils.ApplicationError): ...

class Writer(writers.Writer):
    supported: Tuple[str, ...] = ...
    settings_spec: Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]] = ...
    settings_defaults: Dict[str, str] = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    output: Any = ...
    translator_class: Any = ...
    def __init__(self) -> None: ...
    visitor: Any = ...
    def translate(self) -> None: ...

class XMLTranslator(nodes.GenericNodeVisitor):
    xml_declaration: str = ...
    doctype: str = ...
    generator: str = ...
    xmlparser: xml.sax.xmlreader.XMLReader = ...
    warn: Any = ...
    error: Any = ...
    settings: Any = ...
    indent: str = ...
    newline: str = ...
    level: int = ...
    in_simple: int = ...
    fixed_text: int = ...
    output: List[Any] = ...
    the_handle: docutils_xml.TestXml = ...
    def __init__(self, document: nodes.document) -> None: ...
    simple_nodes: Tuple[nodes.Node, ...] = ...
    def default_visit(self, node: nodes.Node) -> None: ...
    def default_departure(self, node: nodes.Node) -> None: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...

class TestXml(xml.sax.ContentHandler):  # type: ignore
    locator: Any = ...
    def setDocumentLocator(self, locator: Any) -> None: ...



# Stubs for docutils.parsers.rst.directives.misc (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils.nodes import Node
from docutils.parsers.rst import Directive
from typing import Any, Callable, Dict, List, Pattern, TypeVar

N_co = TypeVar('N_co', bound=Node, covariant=True)

__docformat__: str

class Include(Directive):
    required_arguments: int = ...
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    option_spec: Dict[str, Callable[[str], Any]] = ...
    standard_include_path: str = ...
    def run(self) -> List[N_co]: ...

class Raw(Directive):
    required_arguments: int = ...
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    option_spec: Dict[str, Callable[[str], Any]] = ...
    has_content: bool = ...
    def run(self) -> List[N_co]: ...

class Replace(Directive):
    has_content: bool = ...
    def run(self) -> List[N_co]: ...

class Unicode(Directive):
    required_arguments: int = ...
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    option_spec: Dict[str, Callable[[str], Any]] = ...
    comment_pattern: Pattern = ...
    def run(self) -> List[N_co]: ...

class Class(Directive):
    required_arguments: int = ...
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    has_content: bool = ...
    def run(self) -> List[N_co]: ...

class Role(Directive):
    has_content: bool = ...
    argument_pattern: Pattern = ...
    def run(self) -> List[N_co]: ...

class DefaultRole(Directive):
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    def run(self) -> List[N_co]: ...

class Title(Directive):
    required_arguments: int = ...
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    def run(self) -> List[N_co]: ...

class Date(Directive):
    has_content: bool = ...
    def run(self) -> List[N_co]: ...

class TestDirective(Directive):
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    option_spec: Dict[str, Callable[[str], Any]] = ...
    has_content: bool = ...
    def run(self) -> List[N_co]: ...



# Stubs for docutils.writers.xetex (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes
from docutils.writers import latex2e, xetex
from typing import Any, Dict, List, Sequence, Tuple, Type

__docformat__: str

class Writer(latex2e.Writer):
    supported: Tuple[str, ...] = ...
    default_template: str = ...
    default_preamble: str = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    settings_spec: Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]] = ...
    translator_class: Type[xetex.XeLaTeXTranslator] = ...
    def __init__(self) -> None: ...

class Babel(latex2e.Babel):
    language_codes: Dict[str, str] = ...
    language_code: str = ...
    reporter: Any = ...
    language: str = ...
    otherlanguages: Dict[str, Any] = ...
    warn_msg: str = ...
    quote_index: int = ...
    quotes: Tuple[str, str] = ...
    literal_double_quote: str = ...
    def __init__(self, language_code: str, reporter: Any) -> None: ...
    def __call__(self): ...

class XeLaTeXTranslator(latex2e.LaTeXTranslator):
    is_xetex: bool = ...
    def __init__(self, document: nodes.document) -> None: ...



# Stubs for docutils.core (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, Dict, List, Optional, Tuple, Type
from docutils import SettingsSpec
from docutils import nodes
from docutils.frontend import OptionParser, Values
from docutils.io import Input, Output
from docutils.parsers import Parser
from docutils.readers import Reader
from docutils.writers import Writer

__docformat__: str

class Publisher:
    document: nodes.document = ...
    reader: Optional[Reader] = ...
    parser: Optional[Parser] = ...
    writer: Optional[Writer] = ...
    source: Optional[Input] = ...
    source_class: Type[Input] = ...
    destination: Optional[Output] = ...
    destination_class: Type[Output] = ...
    settings: Values = ...
    def __init__(self, reader: Optional[Reader] = ..., parser: Optional[Parser] = ..., writer: Optional[Writer] = ..., source: Optional[Input] = ..., source_class: Type[Input] = ..., destination: Optional[Output] = ..., destination_class: Type[Output] = ..., settings: Optional[Values] = ...) -> None: ...
    def set_reader(self, reader_name: str, parser: Parser, parser_name: str) -> None: ...
    def set_writer(self, writer_name: str) -> None: ...
    def set_components(self, reader_name: str, parser_name: str, writer_name: str) -> None: ...
    def setup_option_parser(self, usage: Optional[Any] = ..., description: Optional[Any] = ..., settings_spec: Optional[SettingsSpec] = ..., config_section: Optional[str] = ..., **defaults) -> OptionParser: ...
    def get_settings(self, usage: Optional[Any] = ..., description: Optional[Any] = ..., settings_spec: Optional[SettingsSpec] = ..., config_section: Optional[str] = ..., **defaults) -> Values: ...
    def process_programmatic_settings(self, settings_spec: SettingsSpec, settings_overrides, config_section: str) -> None: ...
    def process_command_line(self, argv: Optional[List[str]] = ..., usage: Optional[Any] = ..., description: Optional[Any] = ..., settings_spec: Optional[SettingsSpec] = ..., config_section: Optional[str] = ..., **defaults) -> None: ...
    def set_io(self, source_path: Optional[str] = ..., destination_path: Optional[str] = ...) -> None: ...
    def set_source(self, source: Optional[Input] = ..., source_path: Optional[str] = ...) -> None: ...
    def set_destination(self, destination: Optional[Output] = ..., destination_path: Optional[str] = ...) -> None: ...
    def apply_transforms(self) -> None: ...
    def publish(self, argv: Optional[List[str]] = ..., usage: Optional[Any] = ..., description: Optional[Any] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ...) -> Any: ...
    def debugging_dumps(self) -> None: ...
    def report_Exception(self, error: Exception) -> None: ...
    def report_SystemMessage(self, error: Exception) -> None: ...
    def report_UnicodeError(self, error: Exception) -> None: ...

default_usage: str
default_description: str

def publish_cmdline(reader: Optional[Reader] = ..., reader_name: str = ..., parser: Optional[Parser] = ..., parser_name: str = ..., writer: Optional[Writer] = ..., writer_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ..., argv: Optional[List[str]] = ..., usage: Any = ..., description: Any = ...) -> Any: ...
def publish_file(source: Optional[Input] = ..., source_path: Optional[str] = ..., destination: Optional[Output] = ..., destination_path: Optional[str] = ..., reader: Optional[Reader] = ..., reader_name: str = ..., parser: Optional[Parser] = ..., parser_name: str = ..., writer: Optional[Writer] = ..., writer_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ...) -> Any: ...
def publish_string(source: str, source_path: Optional[str] = ..., destination_path: Optional[str] = ..., reader: Optional[Reader] = ..., reader_name: str = ..., parser: Optional[Parser] = ..., parser_name: str = ..., writer: Optional[Writer] = ..., writer_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ...) -> Any: ...
def publish_parts(source: Input, source_path: Optional[str] = ..., source_class: Type[Input] = ..., destination_path: Optional[str] = ..., reader: Optional[Reader] = ..., reader_name: str = ..., parser: Optional[Parser] = ..., parser_name: str = ..., writer: Optional[Writer] = ..., writer_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ...) -> Dict[str, Any]: ...
def publish_doctree(source: str, source_path: Optional[str] = ..., source_class: Type[Input] = ..., reader: Optional[Reader] = ..., reader_name: str = ..., parser: Optional[Parser] = ..., parser_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ...) -> nodes.document: ...
def publish_from_doctree(document: nodes.document, destination_path: Optional[str] = ..., writer: Optional[Writer] = ..., writer_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ...) -> Any: ...
def publish_cmdline_to_binary(reader: Optional[Reader] = ..., reader_name: str = ..., parser: Optional[Parser] = ..., parser_name: str = ..., writer: Optional[Writer] = ..., writer_name: str = ..., settings: Optional[Values] = ..., settings_spec: Optional[SettingsSpec] = ..., settings_overrides: Optional[Any] = ..., config_section: Optional[str] = ..., enable_exit_status: bool = ..., argv: Optional[Any] = ..., usage: Any = ..., description: Any = ..., destination: Optional[Any] = ..., destination_class: Any = ...) -> Any: ...
def publish_programmatically(source_class: Type[Input], source: Any, source_path: str, destination_class: Type[Output], destination: Any, destination_path: str, reader: Reader, reader_name: str, parser: Parser, parser_name: str, writer: Writer, writer_name: str, settings: Values, settings_spec: SettingsSpec, settings_overrides: Any, config_section: str, enable_exit_status: bool) -> Tuple[Any, Publisher]: ...



# Stubs for docutils.writers.pseudoxml (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import writers
from typing import Any, Sequence, Tuple

__docformat__: str

class Writer(writers.Writer):
    supported: Tuple[str, ...] = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    output: str = ...
    def translate(self) -> None: ...
    def supports(self, format: Any) -> bool: ...



# Stubs for docutils (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Any, List, Tuple, Type
from docutils.transforms import Transform

__docformat__: str
__version__: str
__version_info__: Tuple[int, int, int, str, int, bool]
__version_details__: str

class ApplicationError(Exception):
    def __unicode__(self) -> str: ...

class DataError(ApplicationError): ...

class SettingsSpec:
    settings_spec: Any = ...
    settings_defaults: Any = ...
    settings_default_overrides: Any = ...
    relative_path_settings: Any = ...
    config_section: str = ...
    config_section_dependencies: Any = ...

class TransformSpec:
    def get_transforms(self) -> List[Type[Transform]]: ...
    default_transforms: Tuple[Transform] = ...
    unknown_reference_resolvers: Any = ...

class Component(SettingsSpec, TransformSpec):
    component_type: str = ...
    supported: Tuple[str, ...] = ...
    def supports(self, format: str) -> bool: ...



# Stubs for docutils.writers.html4css1 (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes
from docutils.writers import _html_base, html4css1
from typing import Any, Dict, List, Tuple, Type

__docformat__: str

class Writer(_html_base.Writer):
    supported: Tuple[str, ...] = ...
    default_stylesheets: List[str] = ...
    default_stylesheet_dirs: List[str] = ...
    default_template: str = ...
    default_template_path: str = ...
    settings_spec: Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]] = ...
    config_section: str = ...
    parts: Dict[Any, Any] = ...
    translator_class: Type[html4css1.HTMLTranslator] = ...
    def __init__(self) -> None: ...

class HTMLTranslator(_html_base.HTMLTranslator):
    doctype: str = ...
    content_type: str = ...
    content_type_mathml: str = ...
    special_characters: Dict[int, str] = ...
    attribution_formats: Dict[str, Tuple[str, str]] = ...
    def set_first_last(self, node: nodes.Node) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def visit_admonition(self, node: nodes.admonition) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    author_in_authors: bool = ...
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    def is_compactable(self, node: nodes.Node) -> bool: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    in_docinfo: bool = ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    docinfo: List[str] = ...
    body: List[str] = ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_docinfo_item(self, node: nodes.Node, name: str, meta: bool = ...) -> None: ...
    def depart_docinfo_item(self) -> None: ...
    def visit_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    compact_p: Any = ...
    compact_simple: bool = ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    compact_field_list: bool = ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def footnote_backrefs(self, node: nodes.Node) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    object_image_types: Dict[str, str] = ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def should_be_compact_paragraph(self, node: nodes.Node) -> bool: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    in_sidebar: bool = ...
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    in_document_title: int = ...
    def visit_subtitle(self, node: nodes.subtitle) -> None: ...
    subtitle: List[str] = ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...

class SimpleListChecker(_html_base.SimpleListChecker):
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...  # type: ignore
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...  # type: ignore



# Stubs for docutils.writers.odf_odt (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes, writers
from docutils.parsers.rst.directives.html import MetaBody
from docutils.readers import standalone
from docutils.transforms import Transform
from docutils.writers import odf_odt
from types import ModuleType
from typing import Any, AnyStr, Callable, Dict, Iterator, List, Match, Mapping, MutableMapping, MutableSequence, Optional, Pattern, Set, Sequence, Tuple, Type, Union
from xml.etree import ElementTree as etree
import zipfile

VERSION: str
__docformat__: str
IMAGE_NAME_COUNTER: Iterator[int]
WhichElementTree: str
s1: str

class PIL: ...

class _ElementInterface: ...

class _ElementInterfaceWrapper(_ElementInterface):
    def __init__(self, tag: str, attrib: Optional[Mapping] = ...) -> None: ...
    def setparent(self, parent: Any) -> None: ...
    def getparent(self) -> Any: ...

SPACES_PATTERN: Pattern[str]
TABS_PATTERN: Pattern[str]
FILL_PAT1: Pattern[str]
FILL_PAT2: Pattern[str]
TABLESTYLEPREFIX: str
TABLENAMEDEFAULT: str
TABLEPROPERTYNAMES: Tuple[str, ...]
GENERATOR_DESC: str
NAME_SPACE_1: str
CONTENT_NAMESPACE_DICT: Dict[str, str]
CNSD: Dict[str, str]
STYLES_NAMESPACE_DICT: Dict[str, str]
SNSD: Dict[str, str]
MANIFEST_NAMESPACE_DICT: Dict[str, str]
MANNSD: Dict[str, str]
META_NAMESPACE_DICT: Dict[str, str]
METNSD: Dict[str, str]
CONTENT_NAMESPACE_ATTRIB: Dict[str, str]
STYLES_NAMESPACE_ATTRIB: Dict[str, str]
MANIFEST_NAMESPACE_ATTRIB: Dict[str, str]
META_NAMESPACE_ATTRIB: Dict[str, str]

def Element(tag: str, attrib: Optional[Dict[str, Any]] = ..., nsmap: Optional[Any] = ..., nsdict: Mapping[str, str] = ...) -> _ElementInterfaceWrapper: ...
def SubElement(parent: _ElementInterfaceWrapper, tag: str, attrib: Optional[Dict[str, Any]] = ..., nsmap: Optional[Any] = ..., nsdict: Mapping[str, str] = ...) -> _ElementInterfaceWrapper: ...
def fix_ns(tag: str, attrib: Mapping[str, Any], nsdict: Mapping[str, str]) -> Tuple[str, Dict[str, Any]]: ...
def add_ns(tag: str, nsdict: Mapping[str, str] = ...) -> str: ...
def ToString(et: Any) -> str: ...
def escape_cdata(text: str) -> str: ...

WORD_SPLIT_PAT1: Pattern[str]

def split_words(line) -> List[str]: ...

class TableStyle:
    border: Optional[str] = ...
    backgroundcolor: Any = ...
    def __init__(self, border: Optional[str] = ..., backgroundcolor: Optional[Any] = ...) -> None: ...
    def get_border_(self) -> Optional[str]: ...
    border_: Optional[str] = ...
    def set_border_(self, border: Optional[str]) -> None: ...
    def get_backgroundcolor_(self) -> Any: ...
    backgroundcolor_: Any = ...
    def set_backgroundcolor_(self, backgroundcolor: Any) -> None: ...

BUILTIN_DEFAULT_TABLE_STYLE: TableStyle

class ListLevel:
    level: Any = ...
    sibling_level: bool = ...
    nested_level: bool = ...
    def __init__(self, level: int, sibling_level: bool = ..., nested_level: bool = ...) -> None: ...
    def set_sibling(self, sibling_level: bool) -> None: ...
    def get_sibling(self) -> bool: ...
    def set_nested(self, nested_level: bool) -> None: ...
    def get_nested(self) -> bool: ...
    def set_level(self, level: int) -> None: ...
    def get_level(self) -> int: ...

class Writer(writers.Writer):
    MIME_TYPE: str = ...
    EXTENSION: str = ...
    supported: Tuple[str, ...] = ...
    default_stylesheet: str = ...
    default_stylesheet_path: str = ...
    default_template: str = ...
    default_template_path: str = ...
    settings_spec: Tuple[str, Any, Tuple[Tuple[str, List[str], Dict[str, Any]], ...]] = ...
    settings_defaults: Dict[str, str] = ...
    relative_path_settings: Tuple[str, ...] = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    translator_class: Type[odf_odt.ODFTranslator] = ...
    def __init__(self) -> None: ...
    settings: Any = ...
    visitor: odf_odt.ODFTranslator = ...
    output: str = ...
    def translate(self) -> None: ...
    def assemble_my_parts(self) -> None: ...
    def update_stylesheet(self, stylesheet_root: _ElementInterfaceWrapper, language_code: str, region_code: str) -> Tuple[bool, _ElementInterfaceWrapper, Set[_ElementInterfaceWrapper]]: ...
    def write_zip_str(self, zfile: zipfile.ZipFile, name: str, bytes: AnyStr, compress_type: int = ...) -> None: ...
    def store_embedded_files(self, zfile: zipfile.ZipFile) -> None: ...
    def get_settings(self) -> bytes: ...
    def get_stylesheet(self) -> str: ...
    def copy_from_stylesheet(self, outzipfile: zipfile.ZipFile) -> None: ...
    def assemble_parts(self) -> None: ...
    def create_manifest(self) -> str: ...
    def create_meta(self) -> str: ...

class ODFTranslator(nodes.GenericNodeVisitor):
    used_styles: Tuple[str, ...] = ...
    settings: Any = ...
    language_code: str = ...
    language: ModuleType = ...
    format_map: Dict[str, str] = ...
    section_level: int = ...
    section_count: int = ...
    content_tree: etree.ElementTree = ...
    current_element: _ElementInterfaceWrapper = ...
    automatic_styles: _ElementInterfaceWrapper = ...
    body_text_element: _ElementInterfaceWrapper = ...
    paragraph_style_stack: List[str] = ...
    list_style_stack: List[str] = ...
    table_count: int = ...
    column_count: int = ...
    trace_level: int = ...
    optiontablestyles_generated: bool = ...
    field_name: Any = ...
    field_element: Any = ...
    title: Optional[str] = ...
    image_count: int = ...
    image_style_count: int = ...
    image_dict: Dict[str, str] = ...
    embedded_file_list: List[Tuple[str, str]] = ...
    syntaxhighlighting: int = ...
    syntaxhighlight_lexer: str = ...
    header_content: List[etree.Element] = ...
    footer_content: List[etree.Element] = ...
    in_header: bool = ...
    in_footer: bool = ...
    blockstyle: str = ...
    in_table_of_contents: bool = ...
    table_of_content_index_body: Optional[_ElementInterfaceWrapper] = ...
    list_level: int = ...
    def_list_level: int = ...
    footnote_ref_dict: Dict[str, _ElementInterfaceWrapper] = ...
    footnote_list: List[Tuple[nodes.Node, _ElementInterfaceWrapper]] = ...
    footnote_chars_idx: int = ...
    footnote_level: int = ...
    pending_ids: List[str] = ...
    in_paragraph: bool = ...
    found_doc_title: bool = ...
    bumped_list_level_stack: List[ListLevel] = ...
    meta_dict: Dict[str, Any] = ...
    line_block_level: int = ...
    line_indent_level: int = ...
    citation_id: Optional[str] = ...
    style_index: int = ...
    str_stylesheet: str = ...
    str_stylesheetcontent: str = ...
    dom_stylesheet: etree.Element = ...
    table_styles: Dict[str, TableStyle] = ...
    in_citation: bool = ...
    inline_style_count_stack: List[int] = ...
    def __init__(self, document: nodes.document) -> None: ...
    def get_str_stylesheet(self) -> str: ...
    dom_stylesheetcontent: etree.Element = ...
    def retrieve_styles(self, extension: str) -> None: ...
    def extract_table_styles(self, styles_str: str) -> Dict[str, TableStyle]: ...
    def get_property(self, stylenode: Any) -> Any: ...
    def add_doc_title(self) -> None: ...
    def find_first_text_p(self, el: _ElementInterfaceWrapper) -> Optional[_ElementInterfaceWrapper]: ...
    def attach_page_style(self, el: _ElementInterfaceWrapper) -> None: ...
    def rststyle(self, name: str, parameters: Tuple[str] = ...) -> str: ...
    def generate_content_element(self, root: _ElementInterfaceWrapper) -> _ElementInterfaceWrapper: ...
    def setup_page(self) -> str: ...
    def get_dom_stylesheet(self) -> etree.Element: ...
    def setup_paper(self, root_el: _ElementInterfaceWrapper) -> None: ...
    def add_header_footer(self, root_el: _ElementInterfaceWrapper) -> None: ...
    code_none: int = ...
    code_field: int = ...
    code_text: int = ...
    field_pat: Pattern[str] = ...
    def create_custom_headfoot(self, parent: _ElementInterfaceWrapper, text: str, style_name: str, automatic_styles: _ElementInterfaceWrapper) -> None: ...
    def make_field_element(self, parent: _ElementInterfaceWrapper, text: str, style_name: str, automatic_styles: _ElementInterfaceWrapper) -> Optional[_ElementInterfaceWrapper]: ...
    def split_field_specifiers_iter(self, text: str) -> Iterator[Tuple[int, str]]: ...
    def astext(self) -> str: ...
    def content_astext(self) -> str: ...
    def set_title(self, title: str) -> None: ...
    def get_title(self) -> str: ...
    def set_embedded_file_list(self, embedded_file_list: List[Tuple[str, str]]) -> None: ...
    def get_embedded_file_list(self) -> List[Tuple[str, str]]: ...
    def get_meta_dict(self) -> Dict[str, Any]: ...
    def process_footnotes(self) -> None: ...
    def append_child(self, tag: str, attrib: Optional[Dict[str, Any]] = ..., parent: Optional[_ElementInterfaceWrapper] = ...) -> _ElementInterfaceWrapper: ...
    def append_p(self, style: str, text: Optional[Any] = ...) -> _ElementInterfaceWrapper: ...
    def append_pending_ids(self, el: _ElementInterfaceWrapper) -> None: ...
    def set_current_element(self, el: _ElementInterfaceWrapper) -> None: ...
    def set_to_parent(self) -> None: ...
    def generate_labeled_block(self, node: nodes.Node, label: str) -> _ElementInterfaceWrapper: ...
    def generate_labeled_line(self, node: nodes.Node, label: str) -> _ElementInterfaceWrapper: ...
    def encode(self, text: str) -> str: ...
    def dispatch_visit(self, node: nodes.Node) -> None: ...
    def handle_basic_atts(self, node: nodes.Node) -> None: ...
    def default_visit(self, node: nodes.Node) -> None: ...
    def default_departure(self, node: nodes.Node) -> None: ...
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    def depart_contact(self, node: nodes.contact) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def depart_copyright(self, node: nodes.copyright) -> None: ...
    def visit_date(self, node: nodes.date) -> None: ...
    def depart_date(self, node: nodes.date) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def depart_organization(self, node: nodes.organization) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    def depart_status(self, node: nodes.status) -> None: ...
    def visit_revision(self, node: nodes.revision) -> None: ...
    def depart_revision(self, node: nodes.revision) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def depart_version(self, node: nodes.version) -> None: ...
    def visit_attribution(self, node: nodes.attribution) -> None: ...
    def depart_attribution(self, node: nodes.attribution) -> None: ...
    def visit_block_quote(self, node: nodes.block_quote) -> None: ...
    def depart_block_quote(self, node: nodes.block_quote) -> None: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def depart_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_caption(self, node: nodes.caption) -> None: ...
    def depart_caption(self, node: nodes.caption) -> None: ...
    def visit_comment(self, node: nodes.comment) -> None: ...
    def depart_comment(self, node: nodes.comment) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_container(self, node: nodes.container) -> None: ...
    def depart_container(self, node: nodes.container) -> None: ...
    def visit_decoration(self, node: nodes.decoration) -> None: ...
    def depart_decoration(self, node: nodes.decoration) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_document(self, node: nodes.document) -> None: ...
    def depart_document(self, node: nodes.document) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_emphasis(self, node: nodes.emphasis) -> None: ...
    def depart_emphasis(self, node: nodes.emphasis) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_header(self, node: nodes.header) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def visit_footer(self, node: nodes.footer) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    def visit_figure(self, node: nodes.figure) -> None: ...
    def depart_figure(self, node: nodes.figure) -> None: ...
    save_footnote_current: Any = ...
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    footnote_chars: Any = ...
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def depart_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def depart_generated(self, node: nodes.generated) -> None: ...
    def check_file_exists(self, path: str) -> int: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def depart_image(self, node: nodes.image) -> None: ...
    def get_image_width_height(self, node: nodes.Node, attr: str) -> Union[Tuple[None, None], Tuple[float, str], Tuple[str, None]]: ...
    def convert_to_cm(self, size: str) -> Tuple[float, str]: ...
    def get_image_scale(self, node: nodes.Node) -> Union[float, int]: ...
    def get_image_scaled_width_height(self, node: nodes.Node, source: str) -> Tuple[str, str]: ...
    def get_page_width(self) -> float: ...
    def generate_figure(self, node: nodes.Node, source: str, destination: Any, current_element: _ElementInterfaceWrapper) -> Tuple[_ElementInterfaceWrapper, _ElementInterfaceWrapper, _ElementInterfaceWrapper, Optional[str]]: ...
    def generate_image(self, node: nodes.Node, source: str, destination: str, current_element, frame_attrs: Optional[MutableMapping[str, str]] = ...) -> Tuple[_ElementInterfaceWrapper, str]: ...
    def is_in_table(self, node: nodes.Node) -> bool: ...
    def visit_legend(self, node: nodes.legend) -> None: ...
    def depart_legend(self, node: nodes.legend) -> None: ...
    def visit_line_block(self, node: nodes.line_block) -> None: ...
    def depart_line_block(self, node: nodes.line_block) -> None: ...
    def visit_line(self, node: nodes.line) -> None: ...
    def depart_line(self, node: nodes.line) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def visit_inline(self, node: nodes.inline) -> None: ...
    def depart_inline(self, node: nodes.inline) -> None: ...
    def fill_line(self, line: str) -> str: ...
    def fill_func1(self, matchobj: Match[str]) -> str: ...
    def fill_func2(self, matchobj: Match[str]) -> str: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    visit_doctest_block: Callable[[nodes.Node], None] = ...
    depart_doctest_block: Callable[[nodes.Node], None] = ...
    def visit_math(self, node: nodes.math) -> None: ...
    def depart_math(self, node: nodes.math) -> None: ...
    def visit_math_block(self, node: nodes.math_block) -> None: ...
    def depart_math_block(self, node: nodes.math_block) -> None: ...
    def visit_meta(self, node: MetaBody.meta) -> None: ...
    def depart_meta(self, node: MetaBody.meta) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option(self, node: nodes.option) -> None: ...
    def depart_option(self, node: nodes.option) -> None: ...
    def visit_option_string(self, node: nodes.option_string) -> None: ...
    def depart_option_string(self, node: nodes.option_string) -> None: ...
    def visit_option_argument(self, node: nodes.option_argument) -> None: ...
    def depart_option_argument(self, node: nodes.option_argument) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_problematic(self, node: nodes.problematic) -> None: ...
    def depart_problematic(self, node: nodes.problematic) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...
    def depart_raw(self, node: nodes.raw) -> None: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def depart_reference(self, node: nodes.reference) -> None: ...
    def visit_rubric(self, node: nodes.rubric) -> None: ...
    def depart_rubric(self, node: nodes.rubric) -> None: ...
    def visit_section(self, node: nodes.Node, move_ids: int = ...) -> None: ...
    def depart_section(self, node: nodes.section) -> None: ...
    def visit_strong(self, node: nodes.strong) -> None: ...
    def depart_strong(self, node: nodes.strong) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def depart_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def get_table_style(self, node: nodes.Node) -> TableStyle: ...
    current_table_style: _ElementInterfaceWrapper = ...
    table_width: float = ...
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def depart_tgroup(self, node: nodes.tgroup) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    in_thead: bool = ...
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...
    def visit_row(self, node: nodes.row) -> None: ...
    def depart_row(self, node: nodes.row) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    def depart_entry(self, node: nodes.entry) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def depart_target(self, node: nodes.target) -> None: ...
    def visit_title(self, node: nodes.Node, move_ids: int = ..., title_type: str = ...) -> None: ...
    def depart_title(self, node: nodes.title) -> None: ...
    def visit_subtitle(self, node: nodes.Node, move_ids: int = ...) -> None: ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_title_reference(self, node: nodes.title_reference) -> None: ...
    def depart_title_reference(self, node: nodes.title_reference) -> None: ...
    def generate_table_of_content_entry_template(self, el1: _ElementInterfaceWrapper) -> None: ...
    def find_title_label(self, node: nodes.Node, class_type: Type, label_key: str) -> str: ...
    save_current_element: _ElementInterfaceWrapper = ...
    def visit_topic(self, node: nodes.topic) -> None: ...
    def depart_topic(self, node: nodes.topic) -> None: ...
    def update_toc_page_numbers(self, el: _ElementInterfaceWrapper) -> None: ...
    def update_toc_collect(self, el: _ElementInterfaceWrapper, level: int, collection: MutableSequence[Tuple[int, _ElementInterfaceWrapper]]) -> None: ...
    def update_toc_add_numbers(self, collection: Sequence[Tuple[int, _ElementInterfaceWrapper]]) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...
    def depart_transition(self, node: nodes.transition) -> None: ...
    def visit_warning(self, node: nodes.warning) -> None: ...
    def depart_warning(self, node: nodes.warning) -> None: ...
    def visit_attention(self, node: nodes.attention) -> None: ...
    depart_attention: Callable[[nodes.Node], None] = ...
    def visit_caution(self, node: nodes.caution) -> None: ...
    depart_caution: Callable[[nodes.Node], None] = ...
    def visit_danger(self, node: nodes.danger) -> None: ...
    depart_danger: Callable[[nodes.Node], None] = ...
    def visit_error(self, node: nodes.error) -> None: ...
    depart_error: Callable[[nodes.Node], None] = ...
    def visit_hint(self, node: nodes.hint) -> None: ...
    depart_hint: Callable[[nodes.Node], None] = ...
    def visit_important(self, node: nodes.important) -> None: ...
    depart_important: Callable[[nodes.Node], None] = ...
    def visit_note(self, node: nodes.note) -> None: ...
    depart_note: Callable[[nodes.Node], None] = ...
    def visit_tip(self, node: nodes.tip) -> None: ...
    depart_tip: Callable[[nodes.Node], None] = ...
    def visit_admonition(self, node: nodes.admonition) -> None: ...
    depart_admonition: Callable[[nodes.Node], None] = ...
    def generate_admonition(self, node: nodes.Node, label: str, title: Optional[str] = ...) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...

class Reader(standalone.Reader):
    def get_transforms(self) -> List[Type[Transform]]: ...



# Stubs for docutils.parsers (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import Component
from docutils import nodes
from docutils.statemachine import StringList

__docformat__: str

class Parser(Component):
    component_type: str = ...
    config_section: str = ...
    def parse(self, inputstring: str, document: nodes.document) -> None: ...
    inputstring: str = ...
    document: nodes.document = ...
    def setup_parse(self, inputstring: StringList, document: nodes.document) -> None: ...
    def finish_parse(self) -> None: ...

def get_parser_class(parser_name: str) -> Parser: ...



# Stubs for docutils.utils.roman (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Pattern, Tuple

__date__: str
__copyright__: str

class RomanError(Exception): ...
class OutOfRangeError(RomanError): ...
class NotIntegerError(RomanError): ...
class InvalidRomanNumeralError(RomanError): ...

romanNumeralMap: Tuple[Tuple[str, int], ...]

def toRoman(n: int) -> str: ...

romanNumeralPattern: Pattern

def fromRoman(s: str) -> int: ...



# Stubs for docutils.parsers.rst (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

import docutils.parsers
import docutils.statemachine
from docutils import nodes
from docutils.nodes import Node
from docutils.parsers.rst.states import Inliner, RSTState, RSTStateMachine
from docutils.statemachine import StringList
from docutils.transforms import Transform
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Type, TypeVar

N_co = TypeVar('N_co', bound=nodes.Node, covariant=True)

__docformat__: str

class Parser(docutils.parsers.Parser):
    supported: Tuple[str, ...] = ...
    settings_spec: Tuple[str, str, Tuple[str, List[str], Dict[str, Any]]] = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    initial_state: str = ...
    state_classes: Sequence[Type[RSTState]] = ...
    inliner: Inliner = ...
    def __init__(self, rfc2822: bool = ..., inliner: Optional[Inliner] = ...) -> None: ...
    def get_transforms(self) -> List[Type[Transform]]: ...
    statemachine: RSTStateMachine = ...
    def parse(self, inputstring: str, document: nodes.document): ...

class DirectiveError(Exception):
    level: int = ...
    msg: str = ...
    def __init__(self, level: int, message: str) -> None: ...

class Directive:
    required_arguments: int = ...
    optional_arguments: int = ...
    final_argument_whitespace: bool = ...
    option_spec: Dict[str, Callable[[str], Any]] = ...
    has_content: bool = ...
    name: str = ...
    arguments: List[str] = ...
    options: Dict[str, Any] = ...
    content: StringList = ...
    lineno: int = ...
    content_offset: int = ...
    block_text: StringList = ...
    state: RSTState = ...
    state_machine: RSTStateMachine = ...
    def __init__(self, name: str, arguments: List[str], options: Dict[str, Any], content: StringList, lineno: int, content_offset: int, block_text: StringList, state: RSTState, state_machine: RSTStateMachine) -> None: ...
    def run(self) -> List[N_co]: ...
    def directive_error(self, level: int, message: str) -> DirectiveError: ...
    def debug(self, message: str) -> DirectiveError: ...
    def info(self, message: str) -> DirectiveError: ...
    def warning(self, message: str) -> DirectiveError: ...
    def error(self, message: str) -> DirectiveError: ...
    def severe(self, message: str) -> DirectiveError: ...
    def assert_has_content(self) -> None: ...
    def add_name(self, node: Node) -> None: ...

def convert_directive_function(directive_fn: Callable) -> Directive: ...



# Stubs for docutils.utils.math.tex2unichar (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Dict

mathaccent: Dict[str, str]
mathalpha: Dict[str, str]
mathbin: Dict[str, str]
mathclose: Dict[str, str]
mathfence: Dict[str, str]
mathop: Dict[str, str]
mathopen: Dict[str, str]
mathord: Dict[str, str]
mathover: Dict[str, str]
mathradical: Dict[str, str]
mathrel: Dict[str, str]
mathunder: Dict[str, str]
space: Dict[str, str]



# Stubs for docutils.parsers.rst.directives.html (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes
from docutils.nodes import Node
from docutils.parsers.rst import Directive, states
from typing import Dict, List, Tuple, Type, TypeVar

N_co = TypeVar('N_co', bound=Node, covariant=True)

__docformat__: str

class MetaBody(states.SpecializedBody):
    class meta(nodes.Special, nodes.PreBibliographic, nodes.Element): ...
    def field_marker(self, match, context, next_state): ...
    def parsemeta(self, match): ...

class Meta(Directive):
    has_content: bool = ...
    SMkwargs: Dict[str, Tuple[Type[MetaBody]]] = ...
    def run(self) -> List[N_co]: ...



# Stubs for docutils.parsers.rst.roles (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils.nodes import Node, system_message, TextElement
from docutils.parsers.rst import roles
from docutils.parsers.rst.states import Inliner
from docutils.utils import Reporter
from types import ModuleType
from typing import Any, Callable, Dict, List, Tuple, Type, TypeVar, Union

N_co = TypeVar('N_co', bound=Node, covariant=True)
RoleFunction = Callable[[str, str, str, int, Inliner, Dict[str, Any], List[str]], Tuple[List[N_co], List[system_message]]]

__docformat__: str
DEFAULT_INTERPRETED_ROLE: str

def role(role_name: str, language_module: ModuleType, lineno: int, reporter: Reporter) -> Tuple[RoleFunction, List[system_message]]: ...
def register_canonical_role(name: str, role_fn: RoleFunction) -> None: ...
def register_local_role(name: str, role_fn: RoleFunction) -> None: ...
def set_implicit_options(role_fn: RoleFunction) -> None: ...
def register_generic_role(canonical_name: str, node_class: Node) -> None: ...

class GenericRole:
    name: str = ...
    node_class: Node = ...
    def __init__(self, role_name: str, node_class: Type[TextElement]) -> None: ...
    def __call__(self, role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...

class CustomRole:
    name: str = ...
    base_role: Union[RoleFunction, roles.CustomRole] = ...
    options: Dict[str, Any] = ...
    content: List[str] = ...
    supplied_options: Dict[str, Any] = ...
    supplied_content: List[str] = ...
    def __init__(self, role_name: str, base_role: Union[RoleFunction, roles.CustomRole], options: Dict[str, Any] = ..., content: List[str] = ...) -> None: ...
    def __call__(self, role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...

def generic_custom_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def pep_reference_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def rfc_reference_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def raw_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def code_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def math_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, options: Dict[str, Any] = ..., content: List[str] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def unimplemented_role(role: str, rawtext: str, text: str, lineno: int, inliner: Inliner, attributes: Dict[str, Any] = ...) -> Tuple[List[N_co], List[system_message]]: ...
def set_classes(options: Dict[str, str]) -> None: ...



# Stubs for docutils.parsers.rst.languages.en (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from typing import Dict

__docformat__: str
directives: Dict[str, str]
roles: Dict[str, str]



# Stubs for docutils.parsers.null (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import parsers
from docutils.nodes import document
from typing import Sequence, Tuple

class Parser(parsers.Parser):
    supported: Tuple[str, ...] = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    def parse(self, inputstring: str, document: document) -> None: ...



# Stubs for docutils.writers.pep_html (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes
from docutils.writers import html4css1, pep_html
from typing import Dict, List, Sequence, Tuple, Type

__docformat__: str

class Writer(html4css1.Writer):
    default_stylesheet: str = ...
    default_stylesheet_path: str = ...
    default_template: str = ...
    default_template_path: str = ...
    settings_spec: Tuple[str, str, Tuple[Tuple[str, List[str], Dict[str, str]], ...]] = ...
    settings_default_overrides: Dict[str, str] = ...
    relative_path_settings: Tuple[str, ...] = ...
    config_section: str = ...
    config_section_dependencies: Sequence[str] = ...
    translator_class: Type[pep_html.HTMLTranslator] = ...
    def __init__(self) -> None: ...
    pepnum: str = ...
    title: str = ...
    def interpolation_dict(self) -> Dict[str, str]: ...
    def assemble_parts(self) -> None: ...

class HTMLTranslator(html4css1.HTMLTranslator):
    def depart_field_list(self, node: nodes.field_list) -> None: ...



# Stubs for docutils.transforms.references (Python 3.6)
#
# NOTE: This dynamically typed stub was automatically generated by stubgen.

from docutils import nodes
from docutils.transforms import Transform
from docutils.transforms import references
from typing import Callable, Dict, List, Text, Type, Union

__docformat__: str

class PropagateTargets(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore

class AnonymousHyperlinks(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore

class IndirectHyperlinks(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore
    def resolve_indirect_target(self, target: nodes.target) -> None: ...
    def nonexistent_indirect_target(self, target: nodes.target) -> None: ...
    def circular_indirect_reference(self, target: nodes.target) -> None: ...
    def indirect_target_error(self, target: nodes.target, explanation: str) -> None: ...
    def resolve_indirect_references(self, target: nodes.target) -> None: ...

class ExternalTargets(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore

class InternalTargets(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore
    def resolve_reference_ids(self, target: nodes.target): ...

class Footnotes(Transform):
    default_priority: int = ...
    autofootnote_labels: List[str] = ...
    symbols: List[Text] = ...
    def apply(self) -> None: ...  # type: ignore
    def number_footnotes(self, startnum: int) -> int: ...
    def number_footnote_references(self, startnum: int) -> None: ...
    def symbolize_footnotes(self) -> None: ...
    def resolve_footnotes_and_citations(self) -> None: ...
    def resolve_references(self, note: Union[nodes.footnote, nodes.citation], reflist: Union[nodes.footnote_reference, nodes.citation_reference]) -> None: ...

class CircularSubstitutionDefinitionError(Exception): ...

class Substitutions(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore

class TargetNotes(Transform):
    default_priority: int = ...
    classes: List[Type] = ...
    def __init__(self, document: nodes.document, startnode: nodes.Node) -> None: ...
    def apply(self) -> None: ...  # type: ignore
    def make_target_footnote(self, refuri, refs: List[str], notes: Dict[str, nodes.footnote]) -> nodes.footnote: ...

class DanglingReferences(Transform):
    default_priority: int = ...
    def apply(self) -> None: ...  # type: ignore

class DanglingReferencesVisitor(nodes.SparseNodeVisitor):
    document: nodes.document = ...
    unknown_reference_resolvers: List[Callable[[nodes.Node], bool]] = ...
    def __init__(self, document: nodes.document, unknown_reference_resolvers: List[Callable[[nodes.Node], bool]]) -> None: ...
    def unknown_visit(self, node: nodes.Node) -> None: ...
    def visit_reference(self, node: nodes.Node) -> None: ...
    visit_footnote_reference: Callable[[references.DanglingReferencesVisitor, nodes.Node], None] = ...
    visit_citation_reference: Callable[[references.DanglingReferencesVisitor, nodes.Node], None] = ...
